/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CROW_CANVASUID_H
#define CROW_CANVASUID_H

#include "widgets.h"
#include "editors.h"

namespace Crow {

class UIDefinitionCanvasEditorWidget : public EditorWidget {

public:
	UIDefinitionCanvasEditorWidget();

	PGtkActionGroup getActions();
	PGtkUIManager getManager();
	Polytree * getTreeView() { return &treeView; }

protected:
	PGtkActionGroup createActionGroup();
	PGtkUIManager createUIManager(PGtkActionGroup actionGroup);

private:
	Gtk::HBox box;
	Gtk::ScrolledWindow scroll;
	Polytree treeView;

	PGtkUIManager uiManager;
	PGtkActionGroup actionGroup;

};

typedef std::vector<int> UiPath;

struct UIElement : public Object {
	bool remove;
	UiPath path;
	std::string element;
	std::string name;
	std::string action;
	bool expanded;
	UIElement(
		const UiPath & path,
		const std::string & element,
		const std::string & name,
		const std::string & action,
		bool expanded=true
	);
	bool equals(PObject elem);
};

class UIDefinitionCanvasEditor : public CanvasEditor, public TCreateEditorWidget<UIDefinitionCanvasEditorWidget>, public Parser {

public:
	UIDefinitionCanvasEditor();

	void created();
	void load(bool reload);
	void storeSettings();
	void deleted();

protected:

	PUIElement getSelected();
	PUIElement getParent(PUIElement elem);

	void onSelectionChanged();
	void updateActions();
	bool anchestry(PUIElement obj, const std::string & element);
	void updateElementActions(PUIElement parent);
	void popupNewChild(PUIElement parent);
	void onActionNew();
	void onActionAdd();
	void onActionRemove();
	void onActionMoveUp();
	void onActionMoveDown();
	void onActionElement(const char * element);

	std::vector<PUIElement> cut(const UiPath & path, bool siblings);
	void paste(const std::vector<PUIElement> & vec, int shift);
	void updateModel(const std::vector<int> & path);

	std::string toString();
	void fromString(const std::string &);

	void onStartElement(const Glib::ustring & element_name, const Glib::Markup::Parser::AttributeMap & attributes);
	void onEndElement(const Glib::ustring & element_name);

	void updateTree(bool expand=true);
	void setRowValues(PPolyelem elem);
	bool validator(const Glib::ustring & oldtext, const Glib::ustring & newtext);
	void onSetName(PolycellInput * polycell, PUIElement elem);
	void onSetAction(PolycellInput * polycell, PUIElement elem);
	void onPopupActions(PUIElement elem);
	void onActionSelect(PUIElement elem, const std::string & action);
	void onRowCollapsed(const Gtk::TreeModel::iterator &, const Gtk::TreeModel::Path &);
	void onRowExpanded(const Gtk::TreeModel::iterator &, const Gtk::TreeModel::Path &);

	PUIElement getElement(const UiPath & path);
	void setElement(const UiPath & path, PUIElement element);

	void dump(const std::string & str);

private:
	typedef std::map<UiPath, PUIElement> Elements;
	Elements elements;
	UiPath pstack;
	UiPath actionElement;
	bool lock;

};

}

#endif
