/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CROW_CROW_IMPL_H
#define CROW_CROW_IMPL_H

#include "crow.h"
#include "util.h"
#include <guiloader-c++.h>

#define CROW_DESIGNER_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), CROW_TYPE_DESIGNER, CrowDesignerPrivate))

typedef struct _CrowDesignerPrivate CrowDesignerPrivate;

struct _CrowDesignerPrivate {
	Crow::Designer * designer;
};

extern "C" {

void crow_designer_class_init(CrowDesignerClass * meta);
void crow_designer_init(CrowDesigner * obj);
void crow_designer_finalize(GObject * obj);
void crow_designer_emit_actions_updated(CrowActions changed, CrowDesigner * obj);
void crow_designer_emit_status_updated(const Glib::ustring & str, CrowDesigner * obj);
void crow_designer_emit_info_updated(const Glib::ustring & str, CrowDesigner * obj);
void crow_designer_emit_show_url(const Glib::ustring & str, CrowDesigner * obj);
void crow_designer_emit_modal_enter(CrowDesigner * obj);
void crow_designer_emit_modal_leave(CrowDesigner * obj);

}

namespace Crow {

class Designer : virtual public ObjectBase {

public:
	virtual void apply_settings(const Settings & settings) = 0;
	virtual void store_settings(Settings * settings) = 0;
	virtual void clear() = 0;
	virtual void load(const Glib::ustring & data) = 0;
	virtual Glib::ustring save() = 0;
	virtual Glib::ustring save_header() = 0;
	virtual void saved() = 0;
	virtual Gtk::Widget * get_widget() = 0;
	virtual CrowActions get_enabled_actions() = 0;
	virtual void run_action(CrowActions action) = 0;

	typedef sigc::signal<void, CrowActions> SignalActionsUpdated;
	virtual SignalActionsUpdated & signal_actions_updated() = 0;

	typedef sigc::signal<void, Glib::ustring> SignalStatusUpdated;
	virtual SignalStatusUpdated & signal_status_updated() = 0;

	typedef sigc::signal<void, Glib::ustring> SignalInfoUpdated;
	virtual SignalInfoUpdated & signal_info_updated() = 0;

	typedef sigc::signal<void, Glib::ustring> SignalShowUrl;
	virtual SignalShowUrl & signal_show_url() = 0;

	typedef sigc::signal<void> SignalModalEnter;
	virtual SignalModalEnter & signal_modal_enter() = 0;

	typedef sigc::signal<void> SignalModalLeave;
	virtual SignalModalLeave & signal_modal_leave() = 0;


};

class MainInit {

public:
	MainInit() { main= Gtk::Main::instance() ? NULL : new Main; }
	~MainInit() { delete main; }

private:
	
	class Main : public Gtk::Main { };

	Main * main;

};

class CrowInit {

friend class Crow::Object;
friend class Palette;

public:
	static CrowInit * get();

	static void enter(const std::string & msg);
	static void leave();

protected:
	static CrowInit * & ptr();

	CrowInit();
	~CrowInit();

	void init();

	void snapshot(const std::string & msg);
	void check();

	Palette * getPalette() { return palette; }

private:
	MainInit mainInit;
	Gui::Enter guiEnter;
	int refcounter;
	Palette * palette;

	#ifdef CROW_SANDBOX
		typedef std::set<Crow::Object *> Set;
		Set objects;
		std::list<Set> snapshots;
	#else
		int objects;
		std::list<int> snapshots;
	#endif

	std::list<std::string> labels;

	void objectCreated(Crow::Object * object);
	void objectDeleted(Crow::Object * object);

};

}

#endif
