/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CROW_DOCUMENT_H
#define CROW_DOCUMENT_H

#include "util.h"

namespace Crow {

const int GuiNeedsUpgrade = 8;

class Document : public Parser {

public:
	Document();

	const Glib::ustring * getError() { return &error; }

	typedef sigc::slot<NodeTest, PNode> NodeSave;

	Glib::ustring save(Model * model, const Nodes & nodes, const NodeSave & nodeSave);
	bool load(Model * model, const Glib::ustring & data, Nodes & loaded);

protected:
	class Upgrade : public RuntimeError {
	public:
		Upgrade() : RuntimeError("upgrade") { }
	};

	Glib::ustring save(PNode node, int indent);

	StateFlags getNodeState(const std::string & name);
	void onStartElement(const Glib::ustring & name, const AttributeMap & properties);
	void onEndElement(const Glib::ustring & name);
	void onText(const Glib::ustring & text);
	void onPassthrough(const Glib::ustring & text);
	 
	void checkVersion(const std::string & version);

private:
	struct idorder { bool operator () (PNode n1, PNode n2); };

	typedef std::set<PNode, idorder> NodesSorted;

	Model * model;

	NodeSave nodeSave;
	NodeSet nodeSet;

	struct SEntry {
		PNode node;
		Glib::ustring text;
		int leafs;
		SEntry() { leafs= 0; }
		SEntry(const PNode & n, const Glib::ustring & t) { node= n; text= t; leafs= 0; } 
	};

	typedef std::list<SEntry> Stack;
	typedef std::map<std::string, std::string> Names;
	typedef std::list<std::pair<PNode, std::string> > Links;

	Stack stack;
	Names names;
	Links links;
	Nodes objects;

	Glib::ustring error;

};

class GuiUpgrade : public Parser {

public:
	Glib::ustring upgrade(const Glib::ustring & data);

protected:
	struct Node {
		std::string element;
		std::string name;
		std::string type;
		Glib::ustring meta;
		std::list<std::string> flags;
		Glib::ustring text;
		Glib::ustring cdata;
		Node * owner;
		std::list<Node *> domain;
		Node() { owner= NULL; }
	};

	void onStartElement(const Glib::ustring & name, const AttributeMap & properties);
	void onEndElement(const Glib::ustring & name);
	void onText(const Glib::ustring & text);
	void onPassthrough(const Glib::ustring & text);

	Glib::ustring write();
	Glib::ustring write(Node * node, int indent);

	void clean();
	void clean(Node * node);
	Node * find(Node * node, const std::string & name);
	void remove(Node * node);
	void reown(Node * node, Node * owner);

	void upgrade1();
	void upgrade2();
	void upgrade4();
	std::string replaceColons(const std::string & str, const std::string & rep=std::string());
	void upgrade5();
	void upgrade6();
	void upgrade8();

private:
	int version;
	std::list<Node *> stack;
	std::list<Node> model;

};

}

#endif
