/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CROW_FORWARD_H
#define CROW_FORWARD_H

#define CROW_VERSION_GUIXML (9)

#define CROW_LIBRARY

#ifdef G_LOG_DOMAIN
	#undef G_LOG_DOMAIN
#endif
#define G_LOG_DOMAIN "Crow"

namespace Crow {

const char * const CrowProjectHome= "http://crow-designer.sourceforge.net";

using namespace std::rel_ops;

typedef int TypeId;
const TypeId InvalidTypeId = 0;

typedef int EditorId;
const EditorId InvalidEditorId = 0;

typedef unsigned StateFlags;

const StateFlags sfNone          =         0;
const StateFlags sfWeakLink      =   1 <<  0;
const StateFlags sfReadOnly      =   1 <<  3;
const StateFlags sfHidden        =   1 <<  4;
const StateFlags sfNoEditing     =   1 <<  5;
const StateFlags sfLinkOnly      =   1 <<  6;
const StateFlags sfNoUndo        =   1 <<  7;
const StateFlags sfNoSave        =   1 <<  8;
const StateFlags sfOverwrite     =   1 << 10;
const StateFlags sfConstruct     =   1 << 11;
const StateFlags sfAutoDefault   =   1 << 12;
const StateFlags sfTranslatable  =   1 << 14;
const StateFlags sfXml           =   1 << 15;
const StateFlags sfChildDef      =   1 << 16;

const StateFlags sfNodeStateMask = sfWeakLink | sfNoUndo | sfConstruct;

// -----------------------------------------------------------------------------

// canvasuid

struct UIElement;
class UIDefinitionCanvasEditorWidget;

typedef Glib::RefPtr<UIDefinitionCanvasEditorWidget> PUIDefinitionCanvasEditorWidget;
typedef Glib::RefPtr<UIElement> PUIElement;

// canvaswid

class WidgetCanvasEditorWidget;
class WidgetCanvasEditor;

typedef Glib::RefPtr<WidgetCanvasEditorWidget> PWidgetCanvasEditorWidget;
typedef Glib::RefPtr<WidgetCanvasEditor> PWidgetCanvasEditor;

// containers

class Container;

// controller

class Controller;

// crow_impl

class Designer;
typedef std::map<std::string, Glib::ustring> Settings;

// editors

class CanvasEditor;
class Editor;

typedef Glib::RefPtr<CanvasEditor> PCanvasEditor;
typedef Glib::RefPtr<Editor> PEditor;

void RegisterEditors();

// model

class Node;
class Model;
struct Operation;
struct OperationName;
struct OperationMeta;
struct OperationValue;
struct OperationLink;
struct OperationNode;

typedef Glib::RefPtr<Node> PNode;
typedef std::list<PNode> Nodes;
typedef std::set<PNode> NodeSet;
typedef Glib::RefPtr<Operation> POperation;
typedef Glib::RefPtr<OperationName> POperationName;
typedef Glib::RefPtr<OperationMeta> POperationMeta;
typedef Glib::RefPtr<OperationValue> POperationValue;
typedef Glib::RefPtr<OperationLink> POperationLink;
typedef Glib::RefPtr<OperationNode> POperationNode;

// palette

class PaletteEntry;
class EntityEntry;
class Palette;

typedef std::list<std::pair<GType, std::string> > SignalsQuery;

// polytree

class Polycell;
class Polytree;
class PolycellProxy;
class PolycellText;
class PolycellWidget;
class Polyelem;

typedef Glib::RefPtr<PolycellProxy> PPolycellProxy;
typedef Glib::RefPtr<Polycell> PPolycell;
typedef Gtk::TreeModelColumn<PPolycellProxy> PolycellColumn;
typedef Glib::RefPtr<PolycellText> PPolycellText;
typedef Glib::RefPtr<Polyelem> PPolyelem;

// util

class Id;
class Object;
struct Rectangle;
struct Point;

typedef Glib::RefPtr<Object> PObject;
typedef std::vector<Id> IdPath;

// session

class Session;
class SessionManager;

typedef Glib::RefPtr<Session> PSession;
typedef std::map<IdPath, PSession> Sessions;

// value

class CAny;

typedef Glib::RefPtr<CAny> Any;
typedef std::vector<Any> AnyVector;

// views

class Property;
class EntityView;
class GlibObjectView;
class GtkWidgetView;
class GtkContainerView;

typedef std::pair<bool, Property *> NodeTest;

typedef Glib::RefPtr<EntityView> PEntityView;
typedef Glib::RefPtr<GlibObjectView> PGlibObjectView;
typedef Glib::RefPtr<GtkWidgetView> PGtkWidgetView;
typedef Glib::RefPtr<GtkContainerView> PGtkContainerView;

void RegisterViews1();
void RegisterViews2();
void RegisterViews3();

// widgets

class EditorWidget;
class PropertyEditorWidget;
class HierarchyEditorWidget;
class DesignDialog;

typedef Glib::RefPtr<EditorWidget> PEditorWidget;
typedef Glib::RefPtr<PropertyEditorWidget> PPropertyEditorWidget;

// -----------------------------------------------------------------------------

typedef Glib::RefPtr<Glib::Object> PGlibObject;
typedef Glib::RefPtr<Pango::Layout> PPangoLayout;
typedef Glib::RefPtr<Gdk::Pixmap> PGdkPixmap;
typedef Glib::RefPtr<Gdk::Window> PGdkWindow;
typedef Glib::RefPtr<Gdk::Drawable> PGdkDrawable;
typedef Glib::RefPtr<Gdk::GC> PGdkGC;
typedef Glib::RefPtr<Gtk::TreeModel> PGtkTreeModel;
typedef Glib::RefPtr<Gtk::TreeStore> PGtkTreeStore;
typedef Glib::RefPtr<Gtk::ListStore> PGtkListStore;
typedef Glib::RefPtr<Gtk::Widget> PGtkWidget;
typedef Glib::RefPtr<Gtk::UIManager> PGtkUIManager;
typedef Glib::RefPtr<Gtk::ActionGroup> PGtkActionGroup;
typedef Glib::RefPtr<Gtk::Bin> PGtkBin;
typedef Glib::RefPtr<Gtk::Container> PGtkContainer;
typedef Glib::RefPtr<Gtk::AccelGroup> PGtkAccelGroup;
typedef Glib::RefPtr<Gtk::Action> PGtkAction;
typedef Glib::RefPtr<Gtk::Window> PGtkWindow; //???
typedef Glib::RefPtr<Gdk::Pixbuf> PGdkPixbuf;
typedef Glib::RefPtr<Gtk::Style> PGtkStyle;

}

#endif
