/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CROW_VIEWS1_H
#define CROW_VIEWS1_H

#include "views.h"

namespace Crow {

typedef TChildView<RowChild> RowChildView;

class GtkBinView : public TGtkContainerView<BinContainer> {

public:
	GtkBinView();

protected:
	void initDesign();
	void setupContainer();
	void setChildren(Property * property, Any value);

};

class GtkAlignmentView : public GtkBinView, public TCreateInstance<Gtk::Alignment> {

public:
	GtkAlignmentView();

};

class GtkFrameView : public GtkBinView, public TCreateInstance<Gtk::Frame> {

public:
	using TCreateInstance<Gtk::Frame>::getObject;

	GtkFrameView();

protected:
	void setLabelWidgetSet(Property * property, Any value);
	Any getLabel(Property * property);
	void setLabel(Property * property, Any value);
	void configureLabelWidget(Property * property, PEntityView ev);

};

class GtkAspectFrameView : public GtkFrameView {

public:
	GtkAspectFrameView();

protected:
	Any createInstance();

};

class GtkExpanderView : public GtkBinView, public TCreateInstance<Gtk::Expander> {

public:
	using TCreateInstance<Gtk::Expander>::getObject;

	GtkExpanderView();

protected:
	void initInstance();
	void initDesign();
	bool isTopChild(PChild child);
	bool isActiveWidget(Gtk::Widget * widget);
	void setLabelWidgetSet(Property * property, Any value);
	Any getLabel(Property * property);
	void setLabel(Property * property, Any value);
	void configureLabelWidget(Property * property, PEntityView ev);
	void setExpanded(Property * property, Any value);

};

class GtkScrolledWindowView : public GtkBinView, public TCreateInstance<Gtk::ScrolledWindow> {

public:
	using TCreateInstance<Gtk::ScrolledWindow>::getObject;

	GtkScrolledWindowView();

protected:
	ScrolledWindowContainer * createContainer();
	Rectangle getChildArea();
	void configureAdjustment(Property * property, PEntityView view);
	Any getAutoViewport(Property * property);

};

class GtkHandleBoxView : public GtkBinView, public TCreateInstance<Gtk::HandleBox> {

public:
	using TCreateInstance<Gtk::HandleBox>::getObject;

	GtkHandleBoxView();

protected:
	void initInstance();
	void setSnapEdgeSet(Property * property, Any value);

};

class GtkViewportView : public GtkBinView {

public:
	GtkViewportView();

protected:
	Any createInstance();

};

class BoxChildView : public TChildView<BoxChild> {

public:
	BoxChildView();

protected:
	Any getPack(Property * property);
	void setPack(Property * property, Any value);
	Any getOptions(Property * property);
	void setOptions(Property * property, Any value);
	Any getPadding(Property * property);
	void setPadding(Property * property, Any value);

};

class GtkBoxView : public TGtkContainerView<BoxContainer> {

public:
	GtkBoxView();
	PNode createChild(const Place & place, PNode widget);

protected:
	void initDesign();
	void setupContainer();
	void setChildren(Property * property, Any value);
	void setCapacityProperty(Property * property, Any value);
	void setCapacity(int * capS, int * capE);
	void onAsContainerSetup();

};

class GtkHBoxView : public GtkBoxView, public TCreateInstance<Gtk::HBox> {
};

class GtkVBoxView : public GtkBoxView, public TCreateInstance<Gtk::VBox> {
};

class ButtonBoxChildView : public BoxChildView {

public:
	using GlibObjectView::getObject;

	ButtonBoxChildView();

protected:
	Any createInstance();
	Any getSecondary(Property * property);
	void setSecondary(Property * property, Any value);

};

class GtkButtonBoxView : public GtkBoxView {

public:
	using GlibObjectView::getObject;

	GtkButtonBoxView();

protected:
	void initInstance();
	ButtonBoxContainer * createContainer();

};

class GtkHButtonBoxView : public GtkButtonBoxView, public TCreateInstance<Gtk::HButtonBox> {
};

class GtkVButtonBoxView : public GtkButtonBoxView, public TCreateInstance<Gtk::VButtonBox> {
};

class PanedChildView : public TChildView<PanedChild> {

public:
	PanedChildView();

protected:
	Any getResize(Property * property);
	void setResize(Property * property, Any value);
	Any getShrink(Property * property);
	void setShrink(Property * property, Any value);

};

class GtkPanedView : public TGtkContainerView<PanedContainer> {

public:
	GtkPanedView();
	PNode createChild(const Place & place, PNode widget);

protected:
	void initDesign();
	void setupContainer();
	void setChildren(Property * property, Any value);
	void setPositionSet(Property * property, Any value);

};

class GtkHPanedView : public GtkPanedView, public TCreateInstance<Gtk::HPaned> {
};

class GtkVPanedView : public GtkPanedView, public TCreateInstance<Gtk::VPaned> {
};

class NotebookChildView : public TChildView<NotebookChild> {

public:
	NotebookChildView();

protected:
	Any getTabLabel(Property * property);
	void setTabLabel(Property * property, Any value);
	Any getTabLabelWidget(Property * property);
	void setTabLabelWidget(Property * property, Any value);
	Any getMenuLabel(Property * property);
	void setMenuLabel(Property * property, Any value);
	Any getMenuLabelWidget(Property * property);
	void setMenuLabelWidget(Property * property, Any value);
	Any getPackOptions(Property * property);
	void setPackOptions(Property * property, Any value);

};

class GtkNotebookView : public TGtkContainerView<NotebookContainer>, public TCreateInstance<Gtk::Notebook> {

public:
	using TCreateInstance<Gtk::Notebook>::getObject;

	GtkNotebookView();
	PNode createChild(const Place & place, PNode widget);

protected:
	void initDesign();
	void setupContainer();
	void setChildren(Property * property, Any value);
	void setCapacityProperty(Property * property, Any value);
	void setCapacity(int * capacity);
	bool isTopChild(PChild child);
	bool isActiveWidget(Gtk::Widget * widget);
	void onAsContainerSetup();

};

class TableChildView : public TChildView<TableChild> {

public:
	TableChildView();

protected:
	Any getCell(Property * property);
	void setCell(Property * property, Any value);
	Any getCellSpan(Property * property);
	void setCellSpan(Property * property, Any value);
	Any getPaddingX(Property * property);
	void setPaddingX(Property * property, Any value);
	Any getPaddingY(Property * property);
	void setPaddingY(Property * property, Any value);
	Any getXExpand(Property * property);
	void setXExpand(Property * property, Any value);
	Any getXFill(Property * property);
	void setXFill(Property * property, Any value);
	Any getXShrink(Property * property);
	void setXShrink(Property * property, Any value);
	Any getYExpand(Property * property);
	void setYExpand(Property * property, Any value);
	Any getYFill(Property * property);
	void setYFill(Property * property, Any value);
	Any getYShrink(Property * property);
	void setYShrink(Property * property, Any value);

};

class GtkTableView : public TGtkContainerView<TableContainer> {

public:
	GtkTableView();
	PNode createChild(const Place & place, PNode widget);

protected:
	Any createInstance();
	void initDesign();
	void setupContainer();
	void setChildren(Property * property, Any value);
	void setCapacityProperty(Property * property, Any value);
	void setCapacity(int * capX, int * capY);
	void onAsContainerSetup();

};

class XYChildView : public TChildView<XYChild> {

public:
	XYChildView();

protected:
	Any getXY(Property * property);
	void setXY(Property * property, Any value);

};

class XYContainerView : public virtual GtkContainerView {

public:
	XYContainerView();

	bool findFreePlace(const Point & p, FreePlace * fplace);
	FreePlaces findFreePlaces(const Point & p, const Nodes & nodes, const Point & p0);
	PNode createChild(const Place & place, PNode widget);

protected:
	void initDesign();
	virtual PGdkWindow getExposeWindow();
	Point getWidgetPreferredSize(PNode widget);
	bool onExposeEvent(GdkEventExpose * event);
	void onAsContainerSetup();

private:
	sigc::connection expcon;

};

class GtkAdjustmentView : public GlibObjectView {

public:
	GtkAdjustmentView();
	void configureForScrolling();

protected:
	void setRangeProperty(Property * property, Any value);

};

}

#endif
