/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "headers.h"
#include "forward.h"
#include "views2.h"

#include "palette.h"
#include "widgets.h"
#include "controller.h"

namespace Crow {

void RegisterViews2() {

	Palette * palette= Palette::get();

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GdkGravity", 10)).
		addValue("GDK_GRAVITY_NORTH_WEST", Gdk::GRAVITY_NORTH_WEST).
		addValue("GDK_GRAVITY_NORTH", Gdk::GRAVITY_NORTH).
		addValue("GDK_GRAVITY_NORTH_EAST", Gdk::GRAVITY_NORTH_EAST).
		addValue("GDK_GRAVITY_WEST", Gdk::GRAVITY_WEST).
		addValue("GDK_GRAVITY_CENTER", Gdk::GRAVITY_CENTER).
		addValue("GDK_GRAVITY_EAST", Gdk::GRAVITY_EAST).
		addValue("GDK_GRAVITY_SOUTH_WEST", Gdk::GRAVITY_SOUTH_WEST).
		addValue("GDK_GRAVITY_SOUTH", Gdk::GRAVITY_SOUTH).
		addValue("GDK_GRAVITY_SOUTH_EAST", Gdk::GRAVITY_SOUTH_EAST).
		addValue("GDK_GRAVITY_STATIC", Gdk::GRAVITY_STATIC);

	palette->takeEntry(new EnumEntry("GtkWindowType", 2)).
		addValue("GTK_WINDOW_TOPLEVEL", Gtk::WINDOW_TOPLEVEL).
		addValue("GTK_WINDOW_POPUP", Gtk::WINDOW_POPUP);

	palette->takeEntry(new EnumEntry("GdkWindowTypeHint", 8)).
		addValue("GDK_WINDOW_TYPE_HINT_NORMAL", Gdk::WINDOW_TYPE_HINT_NORMAL).
		addValue("GDK_WINDOW_TYPE_HINT_DIALOG", Gdk::WINDOW_TYPE_HINT_DIALOG).
		addValue("GDK_WINDOW_TYPE_HINT_MENU", Gdk::WINDOW_TYPE_HINT_MENU).
		addValue("GDK_WINDOW_TYPE_HINT_TOOLBAR", Gdk::WINDOW_TYPE_HINT_TOOLBAR).
		addValue("GDK_WINDOW_TYPE_HINT_SPLASHSCREEN", Gdk::WINDOW_TYPE_HINT_SPLASHSCREEN).
		addValue("GDK_WINDOW_TYPE_HINT_UTILITY", Gdk::WINDOW_TYPE_HINT_UTILITY).
		addValue("GDK_WINDOW_TYPE_HINT_DOCK", Gdk::WINDOW_TYPE_HINT_DOCK).
		addValue("GDK_WINDOW_TYPE_HINT_DESKTOP", Gdk::WINDOW_TYPE_HINT_DESKTOP);

	palette->takeEntry(new EnumEntry("GtkWindowPosition", 5)).
		addValue("GTK_WIN_POS_NONE", Gtk::WIN_POS_NONE).
		addValue("GTK_WIN_POS_CENTER", Gtk::WIN_POS_CENTER).
		addValue("GTK_WIN_POS_MOUSE", Gtk::WIN_POS_MOUSE).
		addValue("GTK_WIN_POS_CENTER_ALWAYS", Gtk::WIN_POS_CENTER_ALWAYS).
		addValue("GTK_WIN_POS_CENTER_ON_PARENT", Gtk::WIN_POS_CENTER_ON_PARENT);

	palette->takeEntry(new EnumEntry("GtkOrientation", 2)).
		addValue("GTK_ORIENTATION_HORIZONTAL", Gtk::ORIENTATION_HORIZONTAL).
		addValue("GTK_ORIENTATION_VERTICAL", Gtk::ORIENTATION_VERTICAL);

	palette->takeEntry(new EnumEntry("GtkToolbarStyle", 4)).
		addValue("GTK_TOOLBAR_ICONS", Gtk::TOOLBAR_ICONS).
		addValue("GTK_TOOLBAR_TEXT", Gtk::TOOLBAR_TEXT).
		addValue("GTK_TOOLBAR_BOTH", Gtk::TOOLBAR_BOTH).
		addValue("GTK_TOOLBAR_BOTH_HORIZ", Gtk::TOOLBAR_BOTH_HORIZ);

	palette->takeEntry(new TWidgetEntry<GtkWindowView>("GtkWindow", GTK_TYPE_WINDOW, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new EnumEntry("GtkResponseType", 11)).
		addValue("GTK_RESPONSE_NONE", Gtk::RESPONSE_NONE).
		addValue("GTK_RESPONSE_REJECT", Gtk::RESPONSE_REJECT).
		addValue("GTK_RESPONSE_ACCEPT", Gtk::RESPONSE_ACCEPT).
		addValue("GTK_RESPONSE_DELETE_EVENT", Gtk::RESPONSE_DELETE_EVENT).
		addValue("GTK_RESPONSE_OK", Gtk::RESPONSE_OK).
		addValue("GTK_RESPONSE_CANCEL", Gtk::RESPONSE_CANCEL).
		addValue("GTK_RESPONSE_CLOSE", Gtk::RESPONSE_CLOSE).
		addValue("GTK_RESPONSE_YES", Gtk::RESPONSE_YES).
		addValue("GTK_RESPONSE_NO", Gtk::RESPONSE_NO).
		addValue("GTK_RESPONSE_APPLY", Gtk::RESPONSE_APPLY).
		addValue("GTK_RESPONSE_HELP", Gtk::RESPONSE_HELP);

	palette->takeEntry(new TObjectEntry<DialogButtonEntryView>("CrowDialogButtonEntry", G_TYPE_INVALID));

	palette->takeEntry(new TWidgetEntry<GtkDialogView>("GtkDialog", GTK_TYPE_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new TWidgetEntry<GtkColorSelectionDialogView>("GtkColorSelectionDialog", GTK_TYPE_COLOR_SELECTION_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new EnumEntry("GtkFileChooserAction", 4)).
		addValue("GTK_FILE_CHOOSER_ACTION_OPEN", Gtk::FILE_CHOOSER_ACTION_OPEN).
		addValue("GTK_FILE_CHOOSER_ACTION_SAVE", Gtk::FILE_CHOOSER_ACTION_SAVE).
		addValue("GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER", Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER).
		addValue("GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER", Gtk::FILE_CHOOSER_ACTION_CREATE_FOLDER);

	palette->takeEntry(new TObjectEntry<GtkFileFilterView>("GtkFileFilter", GTK_TYPE_FILE_FILTER));

	palette->takeEntry(new TWidgetEntry<GtkFileChooserDialogView>("GtkFileChooserDialog", GTK_TYPE_FILE_CHOOSER_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new TWidgetEntry<GtkFontSelectionDialogView>("GtkFontSelectionDialog", GTK_TYPE_FONT_SELECTION_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GtkPackDirection", 4)).
		addValue("GTK_PACK_DIRECTION_LTR", GTK_PACK_DIRECTION_LTR).
		addValue("GTK_PACK_DIRECTION_RTL", GTK_PACK_DIRECTION_RTL).
		addValue("GTK_PACK_DIRECTION_TTB", GTK_PACK_DIRECTION_TTB).
		addValue("GTK_PACK_DIRECTION_BTT", GTK_PACK_DIRECTION_BTT);

	palette->takeEntry(new TWidgetEntry<GtkMenuBarView>("GtkMenuBar", GTK_TYPE_MENU_BAR, TAB_UI));

	palette->takeEntry(new TWidgetEntry<GtkToolbarView>("GtkToolbar", GTK_TYPE_TOOLBAR, TAB_UI));

	palette->takeEntry(new TWidgetEntry<GtkStatusbarView>("GtkStatusbar", GTK_TYPE_STATUSBAR, TAB_UI));

	palette->takeEntry(new TObjectEntry<TooltipsEntryView>("CrowTooltipsEntry", G_TYPE_INVALID));

	palette->takeEntry(new TObjectEntry<GtkTooltipsView>("GtkTooltips", GTK_TYPE_TOOLTIPS, 1)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GlibObject");

	palette->takeEntry(new TObjectEntry<GtkActionGroupView>("GtkActionGroup", GTK_TYPE_ACTION_GROUP, -2)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GlibObject");

	palette->takeEntry(new TObjectEntry<GtkActionView>("GtkAction", GTK_TYPE_ACTION));

	palette->takeEntry(new TObjectEntry<GtkToggleActionView>("GtkToggleAction", GTK_TYPE_TOGGLE_ACTION));

	palette->takeEntry(new TObjectEntry<GtkRadioActionView>("GtkRadioAction", GTK_TYPE_RADIO_ACTION));

	palette->takeEntry(new TObjectEntry<GtkUIManagerView>("GtkUIManager", GTK_TYPE_UI_MANAGER, -1)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GlibObject");

	palette->takeEntry(new TObjectEntry<UIDefinitionView>("CrowUIDefinition", G_TYPE_INVALID, -2)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|CrowUIDefinition");

	palette->takeEntry(new TObjectEntry<RadioGroupView>("CrowRadioGroup", G_TYPE_INVALID, -3)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GlibObject");

	// -----------------------------------

	palette->takeEntry(new EnumEntry("PangoEllipsizeMode", 4)).
		addValue("PANGO_ELLIPSIZE_NONE", Pango::ELLIPSIZE_NONE).
		addValue("PANGO_ELLIPSIZE_START", Pango::ELLIPSIZE_START).
		addValue("PANGO_ELLIPSIZE_MIDDLE", Pango::ELLIPSIZE_MIDDLE).
		addValue("PANGO_ELLIPSIZE_END", Pango::ELLIPSIZE_END);

	palette->takeEntry(new EnumEntry("GtkJustification", 4)).
		addValue("GTK_JUSTIFY_LEFT", Gtk::JUSTIFY_LEFT).
		addValue("GTK_JUSTIFY_RIGHT", Gtk::JUSTIFY_RIGHT).
		addValue("GTK_JUSTIFY_CENTER", Gtk::JUSTIFY_CENTER).
		addValue("GTK_JUSTIFY_FILL", Gtk::JUSTIFY_FILL);

	palette->takeEntry(new EnumEntry("PangoWrapMode", 3)).
		addValue("PANGO_WRAP_WORD", Pango::WRAP_WORD).
		addValue("PANGO_WRAP_CHAR", Pango::WRAP_CHAR).
		addValue("PANGO_WRAP_WORD_CHAR", Pango::WRAP_WORD_CHAR);

	palette->takeEntry(new TWidgetEntry<GtkLabelView>("GtkLabel", GTK_TYPE_LABEL, TAB_STANDARD));

	palette->takeEntry(new EnumEntry("GtkReliefStyle", 3)).
		addValue("GTK_RELIEF_NORMAL", Gtk::RELIEF_NORMAL).
		addValue("GTK_RELIEF_HALF", Gtk::RELIEF_HALF).
		addValue("GTK_RELIEF_NONE", Gtk::RELIEF_NONE);

	palette->takeEntry(new TWidgetEntry<GtkButtonView>("GtkButton", GTK_TYPE_BUTTON, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkToggleButtonView>("GtkToggleButton", GTK_TYPE_TOGGLE_BUTTON, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkCheckButtonView>("GtkCheckButton", GTK_TYPE_CHECK_BUTTON, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkRadioButtonView>("GtkRadioButton", GTK_TYPE_RADIO_BUTTON, TAB_STANDARD));

}

// GtkWindowView ---------------------------------------------------------------

GtkWindowView::GtkWindowView() {
	addAppPaintableProperty();
	addEventProperties();
	findProperty("visible")->
		setInertDefault(CAny::createBool(false)).
		setFlag(sfReadOnly);
	addInertProperty("accept-focus", prScalar, "bool", CAny::createBool(true));
	addInertProperty("decorated", prScalar, "bool", CAny::createBool(true));
	addInertProperty("default-size", prScalar, "CrowPoint", CAny::createPoint(Point(-1, -1)));
	addInertProperty("focus-on-map", prScalar, "bool", CAny::createBool(true));
	addInertProperty("gravity", prScalar, "GdkGravity", CAny::createEnum("GdkGravity", Gdk::GRAVITY_NORTH_WEST));
	addProperty("icon-name", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string");
	addInertProperty("modal", prScalar, "bool", CAny::createBool(false));
	addInertProperty("resizable", prScalar, "bool", CAny::createBool(true));
	addInertProperty("role", prScalar, "string", CAny::createString(""));
	addInertProperty("skip-pager-hint", prScalar, "bool", CAny::createBool(false));
	addInertProperty("skip-taskbar-hint", prScalar, "bool", CAny::createBool(false));
	addProperty("title", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addInertProperty("type", prScalar, "GtkWindowType", CAny::createEnum("GtkWindowType", Gtk::WINDOW_TOPLEVEL));
	addInertProperty("type-hint", prScalar, "GdkWindowTypeHint", CAny::createEnum("GdkWindowTypeHint", Gdk::WINDOW_TYPE_HINT_NORMAL));
	addInertProperty("window-position", prScalar, "GtkWindowPosition", CAny::createEnum("GtkWindowPosition", Gtk::WIN_POS_NONE));
	addInertProperty("accel-groups", prVector, "GtkUIManager").
		setElementInsert(sigc::mem_fun(*static_cast<GlibObjectView *>(this), &GlibObjectView::insertObject)).
		setFlag(sfLinkOnly);
	addInertProperty("transient-for", prEntity, "GtkWindow").
		setFlag(sfLinkOnly | sfWeakLink);
	addInertProperty("default", prEntity, "GtkWidget").
		setFlag(sfLinkOnly | sfWeakLink);
	addInertProperty("urgency-hint", prScalar, "bool", CAny::createBool(false));
	addInertProperty("deletable", prScalar, "bool", CAny::createBool(true));
}

Gtk::Widget * GtkWindowView::createDesignWindow() {
	Gtk::Widget * window= new DesignWindow();
	SetData(window, "CrowTypeHint", "GtkWindow");
	return window;
}

Any GtkWindowView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr(createDesignWindow()));
}

void GtkWindowView::initDesign() {
	GtkBinView::initDesign();
	getObject()->set_property("title", getDesignLabel());
}

// DialogButtonEntryView -------------------------------------------------------

DialogButtonEntry::DialogButtonEntry() {
	SetData(this, "CrowTypeHint", "CrowDialogButtonEntry");
	response= Gtk::RESPONSE_NONE;
	secondary= false;
}

DialogButtonEntryView::DialogButtonEntryView() {
	addProperty("widget", prEntity, "GtkWidget").
		setValueGet(sigc::mem_fun(*this, &DialogButtonEntryView::getWidget)).
		setValueSet(sigc::mem_fun(*this, &DialogButtonEntryView::setWidget));
	addProperty("response", prScalar, "GtkResponseType", CAny::createEnum("GtkResponseType", Gtk::RESPONSE_NONE)).
		setValueGet(sigc::mem_fun(*this, &DialogButtonEntryView::getResponse)).
		setValueSet(sigc::mem_fun(*this, &DialogButtonEntryView::setResponse));
	addProperty("secondary", prScalar, "bool", CAny::createBool(false)).
		setValueGet(sigc::mem_fun(*this, &DialogButtonEntryView::getSecondary)).
		setValueSet(sigc::mem_fun(*this, &DialogButtonEntryView::setSecondary));
}

Any DialogButtonEntryView::getWidget(Property * property) {
	return CAny::createObject(getObject()->widget);
}

void DialogButtonEntryView::setWidget(Property * property, Any value) {
	getObject()->widget= PGtkWidget::cast_dynamic(value->getObject());
}

Any DialogButtonEntryView::getResponse(Property * property) {
	return CAny::createEnum("GtkResponseType", getObject()->response);
}

void DialogButtonEntryView::setResponse(Property * property, Any value) {
	getObject()->response= (Gtk::ResponseType)value->getEnum();
}

Any DialogButtonEntryView::getSecondary(Property * property) {
	return CAny::createBool(getObject()->secondary);
}

void DialogButtonEntryView::setSecondary(Property * property, Any value) {
	getObject()->secondary= value->getBool();
}

// GtkDialogView ---------------------------------------------------------------

GtkDialogView::GtkDialogView() {
	addProperty("has-separator", prScalar, "bool", CAny::createBool(true));
	findProperty("type-hint")->
		setInertDefault(CAny::createEnum("GdkWindowTypeHint", Gdk::WINDOW_TYPE_HINT_DIALOG));
	findProperty("window-position")->
		setInertDefault(CAny::createEnum("GtkWindowPosition", Gtk::WIN_POS_CENTER_ON_PARENT));
	findProperty("type")->setFlag(sfReadOnly | sfHidden);
	findProperty("app-paintable")->setFlag(sfReadOnly | sfHidden);
	findProperty("default")->setFlag(sfReadOnly | sfHidden);
	addInertProperty("buttons", prVector, "CrowDialogButtonEntry").
		setFlag(sfOverwrite).
		setValueSet(sigc::mem_fun(*this, &GtkDialogView::setButtons)).
		setElementInsert(sigc::mem_fun(*static_cast<GlibObjectView *>(this), &GtkDialogView::insertEntity)).
		setLabelGet(sigc::mem_fun(*this, &GtkDialogView::getButtonLabel));
	addInertProperty("default-response", prScalar, "GtkResponseType", CAny::createEnum("GtkResponseType", Gtk::RESPONSE_NONE));
}

BinContainer * GtkDialogView::createContainer() {
	return new DesignDialogContainer();
}

Gtk::Widget * GtkDialogView::createDesignWindow() {
	Gtk::Widget * window= new DesignDialog();
	SetData(window, "CrowTypeHint", "GtkDialog");
	return window;
}

void GtkDialogView::setButtons(Property * property, Any value) {
	property->setInert(value);
	clearButtons();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		PDialogButtonEntry entry= vec[i]->getObject<DialogButtonEntry>();
		if(entry && entry->widget)
			addButton(entry);
	}
}

void GtkDialogView::clearButtons() {
	ContainerClear(getObject<DesignDialog>()->getButtons());
}

void GtkDialogView::addButton(PDialogButtonEntry entry) {
	getObject<DesignDialog>()->getButtons()->add(*GetPtr(entry->widget));
	if(entry->secondary)
		getObject<DesignDialog>()->getButtons()->set_child_secondary(*GetPtr(entry->widget));
}

std::string GtkDialogView::getButtonLabel(Property * property, PNode node) {
	PNode resp= getModel()->find(node, "response");
	if(resp) {
		std::string str= resp->getValue()->toString();
		return Glib::ustring(str.substr(13)).lowercase();
	}
	return *node->getName();
}

// GtkDialogViewFinal ----------------------------------------------------------

GtkDialogViewFinal::GtkDialogViewFinal() {
	findProperty("buttons")->setFlag(sfReadOnly | sfHidden);
	findProperty("as-container")->setFlag(sfReadOnly | sfHidden);
	findProperty("focus-chain")->setFlag(sfHidden);
	findProperty("focus-child")->setFlag(sfHidden);
	findProperty("default-response")->setFlag(sfReadOnly | sfHidden);
	findProperty("accel-groups")->setFlag(sfReadOnly | sfHidden);
	findProperty("title")->setFlag(sfAutoDefault);
}

// GtkColorSelectionDialogView -------------------------------------------------

GtkColorSelectionDialogView::GtkColorSelectionDialogView() {
	addProperty("colorsel", prEntity, "GtkColorSelection").
		setValueGet(sigc::mem_fun(*this, &GtkColorSelectionDialogView::getColorsel)).
		setFlag(sfReadOnly);
	findProperty("resizable")->
		setInertDefault(CAny::createBool(false));
}

Gtk::Widget * GtkColorSelectionDialogView::createDesignWindow() {
	Gtk::Widget * window= new DesignColorSelectionDialog();
	SetData(window, "CrowTypeHint", "GtkColorSelectionDialog");
	return window;
}

Any GtkColorSelectionDialogView::getColorsel(Property * property) {
	PGtkWidget widget= MakeRefPtr(getObject<DesignColorSelectionDialog>()->getColorsel());
	return CAny::createObject(widget);
}

// GtkFileFilterView -----------------------------------------------------------

GtkFileFilterView::GtkFileFilterView() {
	addSignalsProperty();
	addProperty("name", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &GtkFileFilterView::getName)).
		setValueSet(sigc::mem_fun(*this, &GtkFileFilterView::setName));
	addInertProperty("add-pixbuf-formats", prScalar, "bool", CAny::createBool(false));
	addInertProperty("patterns", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("*.*")
		));
	addInertProperty("mime-types", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("")
		));
}

Any GtkFileFilterView::createInstance() {
	Gtk::FileFilter * filter= new Gtk::FileFilter();
	filter->add_pattern("*.*");
	return CAny::createObject(getType(), NewRefPtr(filter));
}

Any GtkFileFilterView::getName(Property * property) {
	return CAny::createString(getObject<Gtk::FileFilter>()->get_name());
}

void GtkFileFilterView::setName(Property * property, Any value) {
	getObject<Gtk::FileFilter>()->set_name(*value->getString());
}

// GtkFileChooserViewBase ------------------------------------------------------

GtkFileChooserViewBase::GtkFileChooserViewBase() {
	addProperty("action", prScalar, "GtkFileChooserAction", CAny::createEnum("GtkFileChooserAction", Gtk::FILE_CHOOSER_ACTION_OPEN));
	addProperty("extra-widget", prEntity, "GtkWidget");
	addInertProperty("file-system-backend", prScalar, "string", CAny::createString(""));
	addProperty("local-only", prScalar, "bool", CAny::createBool(true));
	addProperty("preview-widget", prEntity, "GtkWidget");
	addProperty("preview-widget-active", prScalar, "bool", CAny::createBool(true));
	addProperty("select-multiple", prScalar, "bool", CAny::createBool(false));
	addProperty("show-hidden", prScalar, "bool", CAny::createBool(false));
	addProperty("use-preview-label", prScalar, "bool", CAny::createBool(true));
	addInertProperty("filter", prEntity, "GtkFileFilter").
		setValueSet(sigc::mem_fun(*this, &GtkFileChooserViewBase::setFilter));
	addInertProperty("filters", prVector, "GtkFileFilter").
		setValueSet(sigc::mem_fun(*this, &GtkFileChooserViewBase::setFilters)).
		setElementInsert(sigc::mem_fun(*static_cast<GlibObjectView *>(this), &GlibObjectView::insertObject));
	addProperty("do-overwrite-confirmation", prScalar, "bool", CAny::createBool(false));
}

Glib::RefPtr<Gtk::FileChooser> GtkFileChooserViewBase::getObject() {
	return Glib::RefPtr<Gtk::FileChooser>::cast_dynamic(GlibObjectView::getObject());
}

void GtkFileChooserViewBase::setFilter(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::FileFilter> filter= value->getObject<Gtk::FileFilter>();
	if(filter) {
		clearVectorProperty("filters")->setFlag(sfReadOnly);
		clearFilters();
	} else
		findProperty("filters")->clearFlag(sfReadOnly);
	touch("filters");
	getObject()->property_filter()= GetPtr(filter);
}

void GtkFileChooserViewBase::setFilters(Property * property, Any value) {
	property->setInert(value);
	clearFilters();
	Glib::RefPtr<Gtk::FileChooser> chooser= getObject();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		Glib::RefPtr<Gtk::FileFilter> obj= vec[i]->getObject<Gtk::FileFilter>();
		if(obj)
			chooser->add_filter(*GetPtr(obj));
	}
}

void GtkFileChooserViewBase::clearFilters() {
	Glib::RefPtr<Gtk::FileChooser> chooser= getObject();
	for(;;) {
		GSList * list= gtk_file_chooser_list_filters(chooser->gobj());
		if(list) {
			GtkFileFilter * filter= GTK_FILE_FILTER(list->data);
			gtk_file_chooser_remove_filter(chooser->gobj(), filter);
		} else
			break;
	}
}

bool GtkFileChooserViewBase::isActiveWidget(Gtk::Widget * widget) {
	if(
		widget==getObject()->get_preview_widget() && 
		!getObject()->get_preview_widget_active()
	)
		return false;
	else
		return true;
}

// GtkFileChooserDialogView ----------------------------------------------------

GtkFileChooserDialogView::GtkFileChooserDialogView() {
	findProperty("buttons")->clearFlag(sfReadOnly | sfHidden);
	findProperty("default-response")->clearFlag(sfReadOnly | sfHidden);
	findProperty("has-separator")->
		setInertDefault(CAny::createBool(false));
}

Glib::RefPtr<Gtk::FileChooser> GtkFileChooserDialogView::getObject() {
	return MakeRefPtr(GlibObjectView::getObject<DesignFileChooserDialog>()->getFileChooser());
}

Gtk::Widget * GtkFileChooserDialogView::createDesignWindow() {
	Gtk::Widget * window= new DesignFileChooserDialog();
	SetData(window, "CrowTypeHint", "GtkFileChooserDialog");
	return window;
}

// GtkFontSelectionDialogView --------------------------------------------------

GtkFontSelectionDialogView::GtkFontSelectionDialogView() {
	addProperty("font-selection", prEntity, "GtkFontSelection").
		setValueGet(sigc::mem_fun(*this, &GtkFontSelectionDialogView::getFontSelection)).
		setFlag(sfReadOnly);
	findProperty("has-separator")->
		setInertDefault(CAny::createBool(false));
}

Gtk::Widget * GtkFontSelectionDialogView::createDesignWindow() {
	Gtk::Widget * window= new DesignFontSelectionDialog();
	SetData(window, "CrowTypeHint", "GtkFontSelectionDialog");
	return window;
}

Any GtkFontSelectionDialogView::getFontSelection(Property * property) {
	PGtkWidget widget= MakeRefPtr(getObject<DesignFontSelectionDialog>()->getFontSelection());
	return CAny::createObject(widget);
}

// GtkStatusbarView ------------------------------------------------------------

GtkStatusbarView::GtkStatusbarView() {
	addProperty("has-resize-grip", prScalar, "bool", CAny::createBool(true));
}

// GtkTooltipsEntryView --------------------------------------------------------

TooltipsEntry::TooltipsEntry() {
	SetData(this, "CrowTypeHint", "CrowTooltipsEntry");
}

TooltipsEntryView::TooltipsEntryView() {
	addProperty("widget", prEntity, "GtkWidget").
		setValueGet(sigc::mem_fun(*this, &TooltipsEntryView::getWidget)).
		setValueSet(sigc::mem_fun(*this, &TooltipsEntryView::setWidget)).
		setFlag(sfLinkOnly | sfWeakLink);
	addProperty("tip-text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &TooltipsEntryView::getTipText)).
		setValueSet(sigc::mem_fun(*this, &TooltipsEntryView::setTipText));
	addProperty("tip-private", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &TooltipsEntryView::getTipPrivate)).
		setValueSet(sigc::mem_fun(*this, &TooltipsEntryView::setTipPrivate));
	addInertProperty("comment", prScalar, "string", CAny::createString(""));
}

Any TooltipsEntryView::getWidget(Property * property) {
	return CAny::createObject(getObject()->widget);
}

void TooltipsEntryView::setWidget(Property * property, Any value) {
	getObject()->widget= PGtkWidget::cast_dynamic(value->getObject());
}

Any TooltipsEntryView::getTipText(Property * property) {
	return CAny::createString(getObject()->tip_text);
}

void TooltipsEntryView::setTipText(Property * property, Any value) {
	getObject()->tip_text= *value->getString();
}

Any TooltipsEntryView::getTipPrivate(Property * property) {
	return CAny::createString(getObject()->tip_private);
}

void TooltipsEntryView::setTipPrivate(Property * property, Any value) {
	getObject()->tip_private= *value->getString();
}

// GtkTooltipsView -------------------------------------------------------------

GtkTooltipsView::GtkTooltipsView() {
	addSignalsProperty();
	addInertProperty("enabled", prScalar, "bool", CAny::createBool(true));
	addInertProperty("tips", prVector, "CrowTooltipsEntry").
		setElementInsert(sigc::mem_fun(*static_cast<GlibObjectView *>(this), &GtkTooltipsView::insertEntity)).
		setLabelGet(sigc::mem_fun(*this, &GtkTooltipsView::getTipLabel));
}

std::string GtkTooltipsView::getTipLabel(Property * property, PNode node) {
	PNode link= getModel()->find(node, "widget");
	if(link) {
		PNode target= link->getLink();
		if(target)
			return *target->getName();
	}
	return *node->getName();
}

// GtkActionGroupView ----------------------------------------------------------

GtkActionGroupView::GtkActionGroupView() {
	addSignalsProperty();
	addInertProperty("name", prScalar, "string", CAny::createString(""));
	addInertProperty("actions", prVector, "GtkAction").
		setValueSet(sigc::mem_fun(*this, &GtkActionGroupView::setActions)).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject)).
		setLabelGet(sigc::mem_fun(*this, &GtkActionGroupView::getActionLabel));
	addProperty("sensitive", prScalar, "bool", CAny::createBool(true));
	addProperty("visible", prScalar, "bool", CAny::createBool(true));
}

Any GtkActionGroupView::createInstance() {
	Glib::ustring name= "temporary_name";
	return CAny::createObject(getType(), Gtk::ActionGroup::create(name));
}

void GtkActionGroupView::doneInstance() {
	clearActions();
	GlibObjectView::doneInstance();
}

void GtkActionGroupView::setActions(Property * property, Any value) {
	property->setInert(value);
	clearActions();
	PGtkActionGroup group= getObject<Gtk::ActionGroup>();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		PGtkAction obj= vec[i]->getObject<Gtk::Action>();
		if(obj) {
			Any accel= getController()->findView(CAny::createObject(obj))->findProperty("accelerator")->getValue();
			if(accel->getString()->empty())
				group->add(obj);
			else
				group->add(obj, Gtk::AccelKey(*accel->getString()));
		}
	}
}

void GtkActionGroupView::clearActions() {
	PGtkActionGroup group= getObject<Gtk::ActionGroup>();
	for(;;) {
		GList * list= gtk_action_group_list_actions(group->gobj());
		if(list) {
			GtkAction * action= GTK_ACTION(list->data);
			gtk_action_group_remove_action(group->gobj(), action);
		} else
			break;
	}
}

std::string GtkActionGroupView::getActionLabel(Property * property, PNode node) {
	PEntityView view= getController()->getView(node);
	if(view)
		return "'" + *view->findProperty("name")->getValue()->getString() + "'";
	else
		return *node->getName();
}

// GtkActionView ---------------------------------------------------------------

GtkActionView::GtkActionView() {
	addSignalsProperty();
	addProperty("name", prScalar, "string").
		setFlag(sfConstruct);
	addProperty("hide-if-empty", prScalar, "bool", CAny::createBool(true));
	addProperty("is-important", prScalar, "bool", CAny::createBool(false));
	addInertProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueSet(sigc::mem_fun(*this, &GtkActionView::setLabel));
	addProperty("sensitive", prScalar, "bool", CAny::createBool(true));
	addProperty("stock-id", prScalar, "GtkStockID", CAny::create(FindType("GtkStockID"), Gtk::StockID())).
		setValueSet(sigc::mem_fun(*this, &GtkActionView::setStockId));
	addProperty("tooltip", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("visible", prScalar, "bool", CAny::createBool(true));
	addProperty("visible-horizontal", prScalar, "bool", CAny::createBool(true));
	addProperty("visible-overflown", prScalar, "bool", CAny::createBool(true));
	addProperty("visible-vertical", prScalar, "bool", CAny::createBool(true));
	addInertProperty("accelerator", prScalar, "string", CAny::createString(""));
	addProperty("icon-name", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string");
	addInertProperty("short-label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
}

Any GtkActionView::createInstance() {
	Glib::ustring name= getPropertyModel<Glib::ustring>("name", "");
	return CAny::createObject(getType(), Gtk::Action::create(name));
}

void GtkActionView::setLabel(Property * property, Any value) {
	property->setInert(value);
	const Glib::ustring & str= *value->getString();
	PGtkAction action= getObject<Gtk::Action>();
	if(str.empty()) {
		g_object_set(action->gobj(), "label", NULL, NULL);
		Gtk::StockID stock= action->property_stock_id();
		action->property_stock_id()= stock;
	} else
		action->property_label()= str;
}

void GtkActionView::setStockId(Property * property, Any value) {
	PGtkAction action= getObject<Gtk::Action>();
	const Gtk::StockID & id= *value->get<Gtk::StockID>();
	action->property_stock_id()= id;
	if(id == Gtk::StockID())
		findProperty("icon-name")->clearFlag(sfReadOnly);
	else {
		findProperty("icon-name")->setFlag(sfReadOnly);
		action->set_property("icon-name", Glib::ustring());
	}
	touch("icon-name");
}

// GtkToggleActionView ---------------------------------------------------------

GtkToggleActionView::GtkToggleActionView() {
	addProperty("draw-as-radio", prScalar, "bool", CAny::createBool(false));
	addProperty("active", prScalar, "bool", CAny::createBool(false));
}

Any GtkToggleActionView::createInstance() {
	Glib::ustring name= getPropertyModel<Glib::ustring>("name", "");
	return CAny::createObject(getType(), Gtk::ToggleAction::create(name));
}

// GtkRadioActionView ----------------------------------------------------------

GtkRadioActionView::GtkRadioActionView() {
	addProperty("value", prScalar, "int", CAny::createInt(0));
	findProperty("draw-as-radio")->
		setDefault(CAny::createBool(true)).
		setFlag(sfReadOnly | sfHidden);
	addInertProperty("group", prEntity, "CrowRadioGroup").
		setFlag(sfWeakLink | sfLinkOnly);
	findProperty("active")->
		setValueSet(sigc::mem_fun(*this, &GtkRadioActionView::setActive));
}

Any GtkRadioActionView::createInstance() {
	Glib::ustring name= getPropertyModel<Glib::ustring>("name", "");
	Gtk::RadioAction::Group group;
	dummy= Gtk::RadioAction::create(group, "");
	Glib::RefPtr<Gtk::RadioAction> action= Gtk::RadioAction::create(group, name);
	action->property_draw_as_radio()= true;
	return CAny::createObject(getType(), action);
}

void GtkRadioActionView::setActive(Property * property, Any value) {
	dummy->set_active(!value->getBool());
	getObject<Gtk::RadioAction>()->set_active(value->getBool());
}

// GtkUIManagerView ------------------------------------------------------------

GtkUIManagerView::GtkUIManagerView() {
	addSignalsProperty();
	addProperty("add-tearoffs", prScalar, "bool", CAny::createBool(false));
	addInertProperty("action-groups", prVector, "GtkActionGroup").
		setFlag(sfLinkOnly).
		setValueSet(sigc::mem_fun(*this, &GtkUIManagerView::setActionGroups)).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject));
	addInertProperty("ui-definitions", prVector, "CrowUIDefinition").
		setFlag(sfLinkOnly).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject));
}

Gtk::UIManager::ui_merge_id GtkUIManagerView::getMergeId(int index) {
	Check(0 <= index && index < (int)mergeIds.size());
	return mergeIds[index];
}

Any GtkUIManagerView::createInstance() {
	PGtkUIManager manager= Gtk::UIManager::create();
	return CAny::createObject(getType(), manager);
}

void GtkUIManagerView::setActionGroups(Property * property, Any value) {
	property->setInert(value);
	PGtkUIManager manager= getObject<Gtk::UIManager>();
	for(;;) {
		GList * list= gtk_ui_manager_get_action_groups(manager->gobj());
		if(list) {
			GtkActionGroup * group= GTK_ACTION_GROUP(list->data);
			gtk_ui_manager_remove_action_group(manager->gobj(), group);
		} else
			break;
	}
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		PGtkActionGroup obj= vec[i]->getObject<Gtk::ActionGroup>();
		if(obj)
			manager->insert_action_group(obj);
	}
}

// UIDefinitionView ------------------------------------------------------------

UIDefinition::UIDefinition() :
	Glib::ObjectBase(typeid(UIDefinition)),
	propertyUi(*this, "ui", "<ui>\n</ui>\n")
{
	SetData(this, "CrowTypeHint", "CrowUIDefinition");
}

UIDefinitionView::UIDefinitionView() {
	addProperty("ui", prScalar, "string", CAny::createString("<ui>\n</ui>\n")).
		setFlag(sfHidden | sfNoEditing | sfXml);
}

// UIElementView ---------------------------------------------------------------

UIElementView::UIElementView() {
	//setReportDestruction(false);
	addInertProperty("element", prScalar, "string", CAny::createString("")).
		setEditor("element|string");
	addInertProperty("manager", prEntity, "GtkUIManager").
		setFlag(sfWeakLink | sfLinkOnly);
}

Any UIElementView::createInstance() {
	return CAny::createObject(getType(), createStub());
}

// GtkMenuBarView --------------------------------------------------------------

GtkMenuBarView::GtkMenuBarView() {
	addEventProperties();
	addProperty("take-focus", prScalar, "bool", CAny::createBool(true));
	addProperty("child-pack-direction", prScalar, "GtkPackDirection", CAny::createEnum("GtkPackDirection", GTK_PACK_DIRECTION_LTR));
	addProperty("pack-direction", prScalar, "GtkPackDirection", CAny::createEnum("GtkPackDirection", GTK_PACK_DIRECTION_LTR));
}

bool GtkMenuBarView::validWidget(Gtk::Widget * widget) {
	return dynamic_cast<Gtk::MenuBar *>(widget);
}

PGtkWidget GtkMenuBarView::createStub() {
	Gtk::MenuBar * menuBar= new Gtk::MenuBar();
	menuBar->items().push_back(Gtk::Menu_Helpers::MenuElem("File"));
	menuBar->items().push_back(Gtk::Menu_Helpers::MenuElem("Edit"));
	menuBar->items().push_back(Gtk::Menu_Helpers::MenuElem("Help"));
	return NewRefPtr(menuBar);
}

// GtkToolbarView --------------------------------------------------------------

GtkToolbarView::GtkToolbarView() {
	addProperty("orientation", prScalar, "GtkOrientation", CAny::createEnum("GtkOrientation", Gtk::ORIENTATION_HORIZONTAL));
	addProperty("show-arrow", prScalar, "bool", CAny::createBool(true));
	addInertProperty("toolbar-style-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkToolbarView::setToolbarStyleSet));
	addInertGetProperty("toolbar-style", prScalar, "GtkToolbarStyle", CAny::createEnum("GtkToolbarStyle", Gtk::TOOLBAR_ICONS)).
		setFlag(sfOverwrite | sfReadOnly);
	addProperty("tooltips", prScalar, "bool", CAny::createBool(true)).
		setValueGet(sigc::mem_fun(*this, &GtkToolbarView::getTooltips)).
		setValueSet(sigc::mem_fun(*this, &GtkToolbarView::setTooltips));
	addInertProperty("icon-size-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkToolbarView::setIconSizeSet));
	addProperty("icon-size", prScalar, "GtkIconSize", CAny::createEnum("GtkIconSize", Gtk::ICON_SIZE_LARGE_TOOLBAR)).
		setFlag(sfReadOnly);
}

bool GtkToolbarView::validWidget(Gtk::Widget * widget) {
	return dynamic_cast<Gtk::Toolbar *>(widget);
}

PGtkWidget GtkToolbarView::createStub() {
	Gtk::Toolbar * toolbar= new Gtk::Toolbar();
	Gtk::ToolButton * b1= Gtk::manage(new Gtk::ToolButton(Gtk::Stock::NEW));
	Gtk::ToolButton * b2= Gtk::manage(new Gtk::ToolButton(Gtk::Stock::OPEN));
	Gtk::ToolButton * b3= Gtk::manage(new Gtk::ToolButton(Gtk::Stock::SAVE));
	b1->show();
	b2->show();
	b3->show();
	toolbar->append(*b1);
	toolbar->append(*b2);
	toolbar->append(*b3);
	return NewRefPtr(toolbar);
}

void GtkToolbarView::setToolbarStyleSet(Property * property, Any value) {
	property->setInert(value);
	if(value->getBool()) {
		findProperty("toolbar-style")->clearFlag(sfReadOnly);
		getObject<Gtk::Toolbar>()->set_toolbar_style(Gtk::TOOLBAR_ICONS);
	} else {
		findProperty("toolbar-style")->
			setFlag(sfReadOnly).
			setInert(CAny::createEnum("GtkToolbarStyle", Gtk::TOOLBAR_ICONS));
		getObject<Gtk::Toolbar>()->unset_toolbar_style();
	}
	touch("toolbar-style");
}

Any GtkToolbarView::getTooltips(Property * property) {
	return CAny::createBool(getObject<Gtk::Toolbar>()->get_tooltips());
}

void GtkToolbarView::setTooltips(Property * property, Any value) {
	getObject<Gtk::Toolbar>()->set_tooltips(value->getBool());
}

void GtkToolbarView::setIconSizeSet(Property * property, Any value) {
	property->setInert(value);
	bool set= value->getBool();
	getObject()->set_property("icon-size", Gtk::ICON_SIZE_LARGE_TOOLBAR);
	getObject()->set_property("icon-size-set", set); 
	if(set)
		findProperty("icon-size")->clearFlag(sfReadOnly);
	else
		findProperty("icon-size")->setFlag(sfReadOnly);
	touch("icon-size");
}

// RadioGroupView --------------------------------------------------------------

RadioGroup::RadioGroup() {
	SetData(this, "CrowTypeHint", "CrowRadioGroup");
}

// GtkMiscView -----------------------------------------------------------------

GtkMiscView::GtkMiscView() {
	addProperty("xalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("xpad", prScalar, "int", CAny::createInt(0));
	addProperty("yalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("ypad", prScalar, "int", CAny::createInt(0));
}

// GtkLabelView ----------------------------------------------------------------

GtkLabelView::GtkLabelView() {
	addProperty("angle", prScalar, "double", CAny::createDouble(0.0));
	addProperty("ellipsize", prScalar, "PangoEllipsizeMode", CAny::createEnum("PangoEllipsizeMode", Pango::ELLIPSIZE_NONE));
	addProperty("justify", prScalar, "GtkJustification", CAny::createEnum("GtkJustification", Gtk::JUSTIFY_LEFT));
	addProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("max-width-chars", prScalar, "int", CAny::createInt(-1));
	addProperty("mnemonic-widget", prEntity, "GtkWidget").
		setFlag(sfLinkOnly | sfWeakLink);
	addProperty("selectable", prScalar, "bool", CAny::createBool(false));
	addProperty("single-line-mode", prScalar, "bool", CAny::createBool(false));
	addProperty("use-markup", prScalar, "bool", CAny::createBool(false));
	addProperty("use-underline", prScalar, "bool", CAny::createBool(false));
	addProperty("wrap-mode", prScalar, "PangoWrapMode", CAny::createEnum("PangoWrapMode", Pango::WRAP_WORD));
	addProperty("wrap", prScalar, "bool", CAny::createBool(false));
}

void GtkLabelView::initDesign() {
	getObject()->set_label(getDesignLabel());
}

// GtkButtonViewBase -----------------------------------------------------------

GtkButtonViewBase::GtkButtonViewBase() {
	addReceivesDefaultProperty(true);
	addCanFocusProperty(true);
	addCanDefaultProperty(false);
	addProperty("focus-on-click", prScalar, "bool", CAny::createBool(true));
	addProperty("relief", prScalar, "GtkReliefStyle", CAny::createEnum("GtkReliefStyle", Gtk::RELIEF_NORMAL));
	addProperty("xalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("yalign", prScalar, "float", CAny::createFloat(0.5f));
}

// GtkButtonView ---------------------------------------------------------------

GtkButtonView::GtkButtonView() {
	addInertProperty("stock-id", prScalar, "GtkStockID", CAny::create(FindType("GtkStockID"), Gtk::StockID())).
		setValueSet(sigc::mem_fun(*this, &GtkButtonView::setStockId));
	addInertGetProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addInertProperty("image", prEntity, "GtkWidget").
		setValueSet(sigc::mem_fun(*this, &GtkButtonView::setImage));
	addProperty("use-underline", prScalar, "bool", CAny::createBool(false));
	addProperty("image-position", prScalar, "GtkPositionType", CAny::createEnum("GtkPositionType", Gtk::POS_LEFT));
}

void GtkButtonView::initDesign() {
	findProperty("label")->setValue(CAny::createString(getDesignLabel()));
}

void GtkButtonView::setupContainer() {
	if(isAsContainer()) {
		resetButton();
		BinRemove(GetPtr(getObject()));
		GtkBinView::setupContainer();
	} else {
		BinRemove(GetPtr(getObject()));
		resetButton();
	}
}

void GtkButtonView::onAsContainerSetup() {
	setState(isAsContainer() ? bsContainer : bsNormal);
}

void GtkButtonView::resetButton() {
	Gtk::Button * button= GetPtr(getObject());
	if(button->get_use_stock()) {
		button->set_use_stock(false);
		button->property_image()= NULL;
	} else
		setImageWidget(NULL);
	button->set_label("");
	button->set_use_underline(false);
}

void GtkButtonView::setState(ButtonState bs) {

	switch(bs) {

		case bsNormal:
			findProperty("label")->clearFlag(sfReadOnly);
			findProperty("image")->clearFlag(sfReadOnly);
			findProperty("stock-id")->clearFlag(sfReadOnly);
			findProperty("use-underline")->clearFlag(sfReadOnly);
			break;
		
		case bsStock:
			findProperty("label")->setFlag(sfReadOnly).setInert(CAny::createString(""));
			findProperty("image")->setFlag(sfReadOnly).setInert(CAny::createObject());
			findProperty("stock-id")->clearFlag(sfReadOnly);
			findProperty("use-underline")->setFlag(sfReadOnly);
			break;
		
		case bsContainer:
			findProperty("label")->setFlag(sfReadOnly).setInert(CAny::createString(""));
			findProperty("image")->setFlag(sfReadOnly).setInert(CAny::createObject());
			findProperty("stock-id")->setFlag(sfReadOnly).setInert(CAny::create(FindType("GtkStockID"), Gtk::StockID()));
			findProperty("use-underline")->setFlag(sfReadOnly);
			break;		
		
		default:
			Check(false);

	}

	touch("label");
	touch("image");
	touch("stock-id");
	touch("use-underline");
}

void GtkButtonView::setStockId(Property * property, Any value) {
	property->setInert(value);
	Glib::ustring str= value->get<Gtk::StockID>()->get_string();
	setState(str.empty() ? bsNormal : bsStock);
	resetButton();
	if(!str.empty()) {
		getObject()->set_label(str);
		getObject()->set_use_stock(true);
	}
}

void GtkButtonView::setImage(Property * property, Any value) {
	property->setInert(value);
	setImageWidget(GetPtr(value->getObject<Gtk::Widget>()));
}

void GtkButtonView::setImageWidget(Gtk::Widget * widget) {
	Check(getObject()->get_use_stock()==false && (!isAsContainer() || widget==NULL));
	Gtk::Widget * old= getObject()->property_image();
	Gtk::Alignment * bin= dynamic_cast<Gtk::Alignment *>(old);
	if(bin)
		BinRemove(bin);
	if(widget) {
		bin= Gtk::manage(new Gtk::Alignment());
		bin->show();
		bin->add(*widget);
		getObject()->property_image()= bin;
	} else
		getObject()->property_image()= NULL;
}

bool GtkButtonView::isActiveWidget(Gtk::Widget * widget) {
	Gtk::Widget * image= getObject()->property_image();
	Gtk::Alignment * bin= dynamic_cast<Gtk::Alignment *>(image);
	if(bin && widget == bin->get_child())
		return Gtk::Settings::get_default()->property_gtk_button_images();
	else
		return true;
}

// GtkToggleButtonView ---------------------------------------------------------

GtkToggleButtonView::GtkToggleButtonView() {
	setInteractive(true);
	addProperty("active", prScalar, "bool", CAny::createBool(false));
	addProperty("inconsistent", prScalar, "bool", CAny::createBool(false));
}

Any GtkToggleButtonView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr<Gtk::ToggleButton>());
}

// GtkCheckButtonView ----------------------------------------------------------

GtkCheckButtonView::GtkCheckButtonView() {
	addReceivesDefaultProperty(false);
	addProperty("draw-indicator", prScalar, "bool", CAny::createBool(true));
}

Any GtkCheckButtonView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr<Gtk::CheckButton>());
}

// GtkRadioButtonView ----------------------------------------------------------

GtkRadioButtonView::GtkRadioButtonView() {
	addInertProperty("group", prEntity, "CrowRadioGroup").
		setFlag(sfWeakLink | sfLinkOnly);
	findProperty("active")->
		setValueSet(sigc::mem_fun(*this, &GtkRadioButtonView::setActive));
}

Any GtkRadioButtonView::createInstance() {
	Gtk::RadioButton::Group group;
	dummy= NewRefPtr(new Gtk::RadioButton(group));
	Glib::RefPtr<Gtk::RadioButton> button= NewRefPtr(new Gtk::RadioButton(group));
	return CAny::createObject(getType(), button);
}

void GtkRadioButtonView::setActive(Property * property, Any value) {
	dummy->set_active(!value->getBool());
	getObject<Gtk::RadioButton>()->set_active(value->getBool());
}

}
