/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "headers.h"
#include "forward.h"
#include "views3.h"

#include "palette.h"
#include "widgets.h"
#include "controller.h"

namespace Crow {

void RegisterViews3() {

	Palette * palette= Palette::get();

	// -----------------------------------

	palette->takeEntry(new TObjectEntry<GtkEntryCompletionView>("GtkEntryCompletion", GTK_TYPE_ENTRY_COMPLETION));

	palette->takeEntry(new TWidgetEntry<GtkEntryView>("GtkEntry", GTK_TYPE_ENTRY, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkComboBoxView>("GtkComboBox", GTK_TYPE_COMBO_BOX, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkComboBoxEntryView>("GtkComboBoxEntry", GTK_TYPE_COMBO_BOX_ENTRY, TAB_STANDARD));

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GtkSpinButtonUpdatePolicy", 2)).
		addValue("GTK_UPDATE_ALWAYS", Gtk::UPDATE_ALWAYS).
		addValue("GTK_UPDATE_IF_VALID", Gtk::UPDATE_IF_VALID);

	palette->takeEntry(new TWidgetEntry<GtkSpinButtonView>("GtkSpinButton", GTK_TYPE_SPIN_BUTTON, TAB_ADDITIONAL));

	palette->takeEntry(new EnumEntry("GtkUpdateType", 3)).
		addValue("GTK_UPDATE_CONTINUOUS", Gtk::UPDATE_CONTINUOUS).
		addValue("GTK_UPDATE_DISCONTINUOUS", Gtk::UPDATE_DISCONTINUOUS).
		addValue("GTK_UPDATE_DELAYED", Gtk::UPDATE_DELAYED);

	palette->takeEntry(new EnumEntry("GtkSensitivityType", 3)).
		addValue("GTK_SENSITIVITY_AUTO", Gtk::SENSITIVITY_AUTO).
		addValue("GTK_SENSITIVITY_ON", Gtk::SENSITIVITY_ON).
		addValue("GTK_SENSITIVITY_OFF", Gtk::SENSITIVITY_OFF);

	palette->takeEntry(new TWidgetEntry<GtkHScaleView>("GtkHScale", GTK_TYPE_HSCALE, TAB_ADDITIONAL));

	palette->takeEntry(new TWidgetEntry<GtkVScaleView>("GtkVScale", GTK_TYPE_VSCALE, TAB_ADDITIONAL));

	palette->takeEntry(new EnumEntry("GtkProgressBarOrientation", 4)).
		addValue("GTK_PROGRESS_LEFT_TO_RIGHT", Gtk::PROGRESS_LEFT_TO_RIGHT).
		addValue("GTK_PROGRESS_RIGHT_TO_LEFT", Gtk::PROGRESS_RIGHT_TO_LEFT).
		addValue("GTK_PROGRESS_BOTTOM_TO_TOP", Gtk::PROGRESS_BOTTOM_TO_TOP).
		addValue("GTK_PROGRESS_TOP_TO_BOTTOM", Gtk::PROGRESS_TOP_TO_BOTTOM);

	palette->takeEntry(new TWidgetEntry<GtkProgressBarView>("GtkProgressBar", GTK_TYPE_PROGRESS_BAR, TAB_ADDITIONAL));

	palette->takeEntry(new EnumEntry("GtkIconSize", 6)).
		addValue("GTK_ICON_SIZE_MENU", Gtk::ICON_SIZE_MENU).
		addValue("GTK_ICON_SIZE_SMALL_TOOLBAR", Gtk::ICON_SIZE_SMALL_TOOLBAR).
		addValue("GTK_ICON_SIZE_LARGE_TOOLBAR", Gtk::ICON_SIZE_LARGE_TOOLBAR).
		addValue("GTK_ICON_SIZE_BUTTON", Gtk::ICON_SIZE_BUTTON).
		addValue("GTK_ICON_SIZE_DND", Gtk::ICON_SIZE_DND).
		addValue("GTK_ICON_SIZE_DIALOG", Gtk::ICON_SIZE_DIALOG);

	palette->takeEntry(new TWidgetEntry<GtkImageView>("GtkImage", GTK_TYPE_IMAGE, TAB_ADDITIONAL));

	palette->takeEntry(new TWidgetEntry<GtkHSeparatorView>("GtkHSeparator", GTK_TYPE_HSEPARATOR, TAB_ADDITIONAL));

	palette->takeEntry(new TWidgetEntry<GtkVSeparatorView>("GtkVSeparator", GTK_TYPE_VSEPARATOR, TAB_ADDITIONAL));

	palette->takeEntry(new EnumEntry("GtkSizeGroupMode", 4)).
		addValue("GTK_SIZE_GROUP_NONE", Gtk::SIZE_GROUP_NONE).
		addValue("GTK_SIZE_GROUP_HORIZONTAL", Gtk::SIZE_GROUP_HORIZONTAL).
		addValue("GTK_SIZE_GROUP_VERTICAL", Gtk::SIZE_GROUP_VERTICAL).
		addValue("GTK_SIZE_GROUP_BOTH", Gtk::SIZE_GROUP_BOTH);

	palette->takeEntry(new TObjectEntry<GtkSizeGroupView>("GtkSizeGroup", GTK_TYPE_SIZE_GROUP, 1)).
		setPaletteTab(TAB_ADDITIONAL).
		setCanvasEditor("canvas|GlibObject");

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GtkWrapMode", 4)).
		addValue("GTK_WRAP_NONE", Gtk::WRAP_NONE).
		addValue("GTK_WRAP_CHAR", Gtk::WRAP_CHAR).
		addValue("GTK_WRAP_WORD", Gtk::WRAP_WORD).
		addValue("GTK_WRAP_WORD_CHAR", Gtk::WRAP_WORD_CHAR);

	palette->takeEntry(new TWidgetEntry<GtkTextViewView>("GtkTextView", GTK_TYPE_TEXT_VIEW, TAB_VIEWS));

	palette->takeEntry(new EnumEntry("GtkSelectionMode", 4)).
		addValue("GTK_SELECTION_NONE", Gtk::SELECTION_NONE).
		addValue("GTK_SELECTION_SINGLE", Gtk::SELECTION_SINGLE).
		addValue("GTK_SELECTION_BROWSE", Gtk::SELECTION_BROWSE).
		addValue("GTK_SELECTION_MULTIPLE", Gtk::SELECTION_MULTIPLE);

	palette->takeEntry(new TWidgetEntry<GtkIconViewView>("GtkIconView", GTK_TYPE_ICON_VIEW, TAB_VIEWS));

	palette->takeEntry(new TWidgetEntry<GtkCellViewView>("GtkCellView", GTK_TYPE_CELL_VIEW, TAB_VIEWS));

	palette->takeEntry(new EnumEntry("GtkTreeViewGridLines", 4)).
		addValue("GTK_TREE_VIEW_GRID_LINES_NONE", Gtk::TREE_VIEW_GRID_LINES_NONE).
		addValue("GTK_TREE_VIEW_GRID_LINES_HORIZONTAL", Gtk::TREE_VIEW_GRID_LINES_HORIZONTAL).
		addValue("GTK_TREE_VIEW_GRID_LINES_VERTICAL", Gtk::TREE_VIEW_GRID_LINES_VERTICAL).
		addValue("GTK_TREE_VIEW_GRID_LINES_BOTH", Gtk::TREE_VIEW_GRID_LINES_BOTH);

	palette->takeEntry(new TWidgetEntry<GtkTreeViewView>("GtkTreeView", GTK_TYPE_TREE_VIEW, TAB_VIEWS));

	// -----------------------------------

	palette->takeEntry(new TWidgetEntry<GtkColorButtonView>("GtkColorButton", GTK_TYPE_COLOR_BUTTON, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkColorSelectionView>("GtkColorSelection", GTK_TYPE_COLOR_SELECTION, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFileChooserButtonView>("GtkFileChooserButton", GTK_TYPE_FILE_CHOOSER_BUTTON, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFileChooserWidgetView>("GtkFileChooserWidget", GTK_TYPE_FILE_CHOOSER_WIDGET, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFontButtonView>("GtkFontButton", GTK_TYPE_FONT_BUTTON, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkFontSelectionView>("GtkFontSelection", GTK_TYPE_FONT_SELECTION, TAB_SELECTORS));

	// -----------------------------------

	palette->takeEntry(new TWidgetEntry<GtkFixedView>("GtkFixed", GTK_TYPE_FIXED, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkLayoutView>("GtkLayout", GTK_TYPE_LAYOUT, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkHScrollbarView>("GtkHScrollbar", GTK_TYPE_HSCROLLBAR, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkVScrollbarView>("GtkVScrollbar", GTK_TYPE_VSCROLLBAR, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkEventBoxView>("GtkEventBox", GTK_TYPE_EVENT_BOX, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkDrawingAreaView>("GtkDrawingArea", GTK_TYPE_DRAWING_AREA, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<GtkCalendarView>("GtkCalendar", GTK_TYPE_CALENDAR, TAB_SPECIAL));

	palette->takeEntry(new EnumEntry("GtkArrowType", 4)).
		addValue("GTK_ARROW_UP", Gtk::ARROW_UP).
		addValue("GTK_ARROW_DOWN", Gtk::ARROW_DOWN).
		addValue("GTK_ARROW_LEFT", Gtk::ARROW_LEFT).
		addValue("GTK_ARROW_RIGHT", Gtk::ARROW_RIGHT);

	palette->takeEntry(new TWidgetEntry<GtkArrowView>("GtkArrow", GTK_TYPE_ARROW, TAB_SPECIAL));

	palette->takeEntry(new TWidgetEntry<CustomView>("CrowCustom", GTK_TYPE_WIDGET, TAB_SPECIAL));

	// -----------------------------------

	palette->takeEntry(new TWidgetEntry<ButtonView>("CrowButton", GTK_TYPE_WIDGET, TAB_DEBUG)).
		setFlags(efDebug);

	palette->takeEntry(new TWidgetEntry<WindowView>("CrowWindow", GTK_TYPE_WINDOW, TAB_DEBUG)).
		setFlags(efDebug | efTopMaster);

	// -----------------------------------

	palette->takeEntry(new TObjectEntry<GtkStatusIconView>("GtkStatusIcon", GTK_TYPE_STATUS_ICON)).
		setPaletteTab(TAB_SPECIAL).
		setCanvasEditor("canvas|GlibObject");

	palette->takeEntry(new TWidgetEntry<GtkLinkButtonView>("GtkLinkButton", GTK_TYPE_LINK_BUTTON, TAB_ADDITIONAL));

	palette->takeEntry(new TObjectEntry<GtkRecentFilterView>("GtkRecentFilter", GTK_TYPE_RECENT_FILTER));

	palette->takeEntry(new EnumEntry("GtkRecentSortType", 4)).
		addValue("GTK_RECENT_SORT_NONE", Gtk::RECENT_SORT_NONE).
		addValue("GTK_RECENT_SORT_MRU", Gtk::RECENT_SORT_MRU).
		addValue("GTK_RECENT_SORT_LRU", Gtk::RECENT_SORT_LRU).
		addValue("GTK_RECENT_SORT_CUSTOM", Gtk::RECENT_SORT_CUSTOM);

	palette->takeEntry(new TWidgetEntry<GtkRecentChooserWidgetView>("GtkRecentChooserWidget", GTK_TYPE_RECENT_CHOOSER_WIDGET, TAB_SELECTORS));

	palette->takeEntry(new TWidgetEntry<GtkRecentChooserDialogView>("GtkRecentChooserDialog", GTK_TYPE_RECENT_CHOOSER_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

}

// GtkEntryCompletionView ------------------------------------------------------

GtkEntryCompletionView::GtkEntryCompletionView() {
	addSignalsProperty();
	addProperty("inline-completion", prScalar, "bool", CAny::createBool(false));
	addProperty("minimum-key-length", prScalar, "int", CAny::createInt(1));
	addProperty("popup-completion", prScalar, "bool", CAny::createBool(true));
	addInertProperty("strings", prVector, "string").
		setFlag(sfTranslatable).
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("text")
		));
	addProperty("popup-set-width", prScalar, "bool", CAny::createBool(true));
	addProperty("popup-single-match", prScalar, "bool", CAny::createBool(true));
}

Any GtkEntryCompletionView::createInstance() {
	return CAny::createObject(getType(), Gtk::EntryCompletion::create());
}

// GtkEntryView ----------------------------------------------------------------

GtkEntryView::GtkEntryView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("activates-default", prScalar, "bool", CAny::createBool(false));
	addProperty("editable", prScalar, "bool", CAny::createBool(true));
	addProperty("has-frame", prScalar, "bool", CAny::createBool(true));
	addProperty("invisible-char", prScalar, "char").
		setFlag(sfAutoDefault);
	addProperty("max-length", prScalar, "int", CAny::createInt(0));
	addProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("visibility", prScalar, "bool", CAny::createBool(true));
	addProperty("width-chars", prScalar, "int", CAny::createInt(-1));
	addProperty("xalign", prScalar, "float", CAny::createFloat(0));
	addProperty("completion", prEntity, "GtkEntryCompletion").
		setValueGet(sigc::mem_fun(*this, &GtkEntryView::getCompletion)).
		setValueSet(sigc::mem_fun(*this, &GtkEntryView::setCompletion));
	addInertProperty("inner-border-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkEntryView::setInnerBorderSet));
	addInertProperty("inner-border", prScalar, "GtkBorder", CAny::create(FindType("GtkBorder"), Border())).
		setValueSet(sigc::mem_fun(*this, &GtkEntryView::setInnerBorder)).
		setFlag(sfReadOnly);
	addProperty("truncate-multiline", prScalar, "bool", CAny::createBool(false));
}

void GtkEntryView::initDesign() {
	getObject()->set_text(getDesignLabel());
}

Any GtkEntryView::getCompletion(Property * property) {
	return CAny::createObject(getObject()->get_completion());
}

void GtkEntryView::setCompletion(Property * property, Any value) {
	getObject()->set_completion(value->getObject<Gtk::EntryCompletion>());
}

void GtkEntryView::setInnerBorderSet(Property * property, Any value) {
	property->setInert(value);
	Property * P= findProperty("inner-border");
	if(value->getBool()) {
		P->clearFlag(sfReadOnly);
		setInnerBorder(P, P->getDefault());
	} else {
		P->setFlag(sfReadOnly);
		P->setInert(P->getDefault());
		gtk_entry_set_inner_border(getObject()->gobj(), NULL);
	}
	touch("inner-border");
}

void GtkEntryView::setInnerBorder(Property * property, Any value) {
	property->setInert(value);
	Gtk::Border border= *value->get<Border>();
	getObject()->set_inner_border(border);
}

// GtkComboBoxView -------------------------------------------------------------

GtkComboBoxView::GtkComboBoxView() {
	addInertProperty("add-tearoffs", prScalar, "bool", CAny::createBool(false));
	addProperty("focus-on-click", prScalar, "bool", CAny::createBool(true));
	addInertProperty("text-mode", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkComboBoxView::setTextMode));
	addInertProperty("strings", prVector, "string").
		setFlag(sfReadOnly | sfTranslatable).
		setValueSet(sigc::mem_fun(*this, &GtkComboBoxView::setStrings)).
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*static_cast<GlibObjectView *>(this), &GlibObjectView::insertScalar),
			CAny::createString("text")
		));
	addProperty("active", prScalar, "int", CAny::createInt(-1));
	addProperty("tearoff-title", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
}

Any GtkComboBoxView::createInstance() {
	Gtk::Widget * widget= new Gtk::ComboBoxText();
	SetData(widget, "CrowTypeHint", "GtkComboBox");
	return CAny::createObject(getType(), NewRefPtr(widget));
}

void GtkComboBoxView::setTextMode(Property * property, Any value) {
	property->setInert(value);
	if(value->getBool()) {
		findProperty("strings")->clearFlag(sfReadOnly);
	} else {
		clearVectorProperty("strings")->setFlag(sfReadOnly);
		clearStrings();
	}
	touch("strings");
}

void GtkComboBoxView::setStrings(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::ComboBoxText> combo= getObject<Gtk::ComboBoxText>();
	int active= combo->property_active();
	combo->clear_items();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		const Glib::ustring & str= *vec[i]->getString();
		combo->append_text(str);
	}
	combo->property_active()= active;
}

void GtkComboBoxView::clearStrings() {
	getObject<Gtk::ComboBoxText>()->clear_items();
}

// GtkComboBoxEntryView --------------------------------------------------------

GtkComboBoxEntryView::GtkComboBoxEntryView() {
	addProperty("has-frame", prScalar, "bool", CAny::createBool(true));
	addProperty("text", prScalar, "string", CAny::createString("")).
		setValueGet(sigc::mem_fun(*this, &GtkComboBoxEntryView::getText)).
		setValueSet(sigc::mem_fun(*this, &GtkComboBoxEntryView::setText));
	addProperty("entry", prEntity, "GtkEntry").
		setValueGet(sigc::mem_fun(*this, &GtkComboBoxEntryView::getEntry)).
		setViewConfigure(sigc::mem_fun(*this, &GtkComboBoxEntryView::configureEntry)).
		setFlag(sfReadOnly);
	findProperty("active")->
		setValueSet(sigc::mem_fun(*this, &GtkComboBoxEntryView::setActive));
}

Any GtkComboBoxEntryView::createInstance() {
	Gtk::Widget * widget= new Gtk::ComboBoxEntryText();
	SetData(widget, "CrowTypeHint", "GtkComboBoxEntry");
	return CAny::createObject(getType(), NewRefPtr(widget));
}

Any GtkComboBoxEntryView::getEntry(Property * property) {
	return CAny::createObject(MakeRefPtr(getObject<Gtk::ComboBoxEntry>()->get_entry()));
}

Any GtkComboBoxEntryView::getText(Property * property) {
	return CAny::createString(getObject<Gtk::ComboBoxEntry>()->get_entry()->get_text());
}

void GtkComboBoxEntryView::setText(Property * property, Any value) {
	getObject<Gtk::ComboBoxEntry>()->get_entry()->set_text(*value->getString());
}

void GtkComboBoxEntryView::setActive(Property * property, Any value) {
	Gtk::ComboBoxEntry * combo= GetPtr(getObject<Gtk::ComboBoxEntry>());
	int active= combo->property_active();
	getObject<Gtk::ComboBoxEntry>()->property_active()= value->getInt();
	activeChanged(active);
}

void GtkComboBoxEntryView::configureEntry(Property * property, PEntityView view) {
	view->findProperty("has-frame")->setFlag(sfReadOnly | sfHidden);
	view->findProperty("text")->setFlag(sfReadOnly | sfHidden);
}

void GtkComboBoxEntryView::setStrings(Property * property, Any value) {
	property->setInert(value);
	Gtk::ComboBoxEntryText * combo= GetPtr(getObject<Gtk::ComboBoxEntryText>());
	int active= combo->property_active();
	combo->clear_items();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		const Glib::ustring & str= *vec[i]->getString();
		combo->append_text(str);
	}
	combo->property_active()= active;
	activeChanged(active);
}

void GtkComboBoxEntryView::clearStrings() {
	Gtk::ComboBoxEntryText * combo= GetPtr(getObject<Gtk::ComboBoxEntryText>());
	int active= combo->property_active();
	combo->clear_items();
	combo->property_active()= -1;
	activeChanged(active);
}

void GtkComboBoxEntryView::activeChanged(int old) {
	Gtk::ComboBoxEntry * combo= GetPtr(getObject<Gtk::ComboBoxEntry>());
	int actual= combo->property_active(); 
	if(actual != old) {
		if(actual > -1)
			findProperty("text")->setFlag(sfReadOnly);
		else {
			combo->get_entry()->set_text("");
			findProperty("text")->clearFlag(sfReadOnly);
		}
		touch("text");
	}
}

// GtkSpinButtonView -----------------------------------------------------------

GtkSpinButtonView::GtkSpinButtonView() {
	findProperty("text")->setFlag(sfReadOnly | sfHidden);
	addProperty("adjustment", prEntity, "GtkAdjustment").
		setFlag(sfReadOnly).
		setViewConfigure(sigc::mem_fun(*this, &GtkSpinButtonView::configureAdjustment));
	addProperty("climb-rate", prScalar, "double", CAny::createDouble(0));
	addProperty("digits", prScalar, "unsigned", CAny::createUnsigned(0));
	addProperty("numeric", prScalar, "bool", CAny::createBool(false));
	addProperty("snap-to-ticks", prScalar, "bool", CAny::createBool(false));
	addProperty("update-policy", prScalar, "GtkSpinButtonUpdatePolicy", CAny::createEnum("GtkSpinButtonUpdatePolicy", Gtk::UPDATE_ALWAYS));
	addProperty("value", prScalar, "double", CAny::createDouble(0));
	addProperty("wrap", prScalar, "bool", CAny::createBool(false));
}

Any GtkSpinButtonView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr<Gtk::SpinButton>());
}

void GtkSpinButtonView::initDesign() {
}

void GtkSpinButtonView::configureAdjustment(Property * property, PEntityView view) {
	view->findProperty("value")->setFlag(sfReadOnly);
}

// GtkRangeView ----------------------------------------------------------------

GtkRangeView::GtkRangeView() {
	setInteractive(true);
	addProperty("inverted", prScalar, "bool", CAny::createBool(false));
	addProperty("update-policy", prScalar, "GtkUpdateType", CAny::createEnum("GtkUpdateType", Gtk::UPDATE_CONTINUOUS));
	addProperty("adjustment", prEntity, "GtkAdjustment").
		setFlag(sfReadOnly);
	addProperty("lower-stepper-sensitivity", prScalar, "GtkSensitivityType", CAny::createEnum("GtkSensitivityType", Gtk::SENSITIVITY_AUTO));
	addProperty("upper-stepper-sensitivity", prScalar, "GtkSensitivityType", CAny::createEnum("GtkSensitivityType", Gtk::SENSITIVITY_AUTO));
}

// GtkScaleView ----------------------------------------------------------------

GtkScaleView::GtkScaleView() {
	addProperty("digits", prScalar, "int", CAny::createInt(1));
	addProperty("draw-value", prScalar, "bool", CAny::createBool(true));
	addProperty("value-pos", prScalar, "GtkPositionType", CAny::createEnum("GtkPositionType", Gtk::POS_TOP));
	addCanFocusProperty(true);
}

// GtkProgressBarView ----------------------------------------------------------

GtkProgressBarView::GtkProgressBarView() {
	addEventProperties();
	addProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("fraction", prScalar, "double", CAny::createDouble(0));
	addProperty("pulse-step", prScalar, "double", CAny::createDouble(0.1));
	addProperty("orientation", prScalar, "GtkProgressBarOrientation", CAny::createEnum("GtkProgressBarOrientation", Gtk::PROGRESS_LEFT_TO_RIGHT));
	addProperty("ellipsize", prScalar, "PangoEllipsizeMode", CAny::createEnum("PangoEllipsizeMode", Pango::ELLIPSIZE_NONE));
}

// GtkImageView ----------------------------------------------------------------

GtkImageView::GtkImageView() {
	addProperty("icon-name", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string");
	addProperty("icon-size", prScalar, "GtkIconSize", CAny::createEnum("GtkIconSize", Gtk::ICON_SIZE_BUTTON));
	addProperty("pixel-size", prScalar, "int", CAny::createInt(-1));
}

void GtkImageView::initInstance() {
	GtkMiscView::initInstance();
	getObject()->set_from_icon_name("", Gtk::ICON_SIZE_BUTTON);
}

// GtkSizeGroupView ------------------------------------------------------------

GtkSizeGroupView::GtkSizeGroupView() {
	addSignalsProperty();
	addProperty("mode", prScalar, "GtkSizeGroupMode", CAny::createEnum("GtkSizeGroupMode", Gtk::SIZE_GROUP_HORIZONTAL));
	addProperty("ignore-hidden", prScalar, "bool", CAny::createBool(false));
	addInertProperty("widgets", prVector, "GtkWidget").
		setFlag(sfLinkOnly | sfWeakLink).
		setValueSet(sigc::mem_fun(*this, &GtkSizeGroupView::setWidgets)).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject));
}

Any GtkSizeGroupView::createInstance() {
	return CAny::createObject(getType(), Gtk::SizeGroup::create(Gtk::SIZE_GROUP_HORIZONTAL));
}

void GtkSizeGroupView::doneInstance() {
	clearWidgets();
	GlibObjectView::doneInstance();
}

void GtkSizeGroupView::clearWidgets() {
	Glib::RefPtr<Gtk::SizeGroup> sizeGroup= getObject<Gtk::SizeGroup>();
	for(;;) {
		GSList * list= gtk_size_group_get_widgets(sizeGroup->gobj());
		if(list) {
			GtkWidget * widget= GTK_WIDGET(list->data);
			gtk_size_group_remove_widget(sizeGroup->gobj(), widget);
		} else
			break;
	}
}

void GtkSizeGroupView::setWidgets(Property * property, Any value) {
	property->setInert(value);
	clearWidgets();
	Glib::RefPtr<Gtk::SizeGroup> sizeGroup= getObject<Gtk::SizeGroup>();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		PGtkWidget obj= vec[i]->getObject<Gtk::Widget>();
		if(obj)
			sizeGroup->add_widget(*GetPtr(obj));
	}
}

// GtkTextViewView -------------------------------------------------------------

GtkTextViewView::GtkTextViewView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("accepts-tab", prScalar, "bool", CAny::createBool(true));
	addProperty("cursor-visible", prScalar, "bool", CAny::createBool(true));
	addProperty("editable", prScalar, "bool", CAny::createBool(true));
	addProperty("indent", prScalar, "int", CAny::createInt(0));
	addProperty("justification", prScalar, "GtkJustification", CAny::createEnum("GtkJustification", Gtk::JUSTIFY_LEFT));
	addProperty("left-margin", prScalar, "int", CAny::createInt(0));
	addProperty("overwrite", prScalar, "bool", CAny::createBool(false));
	addProperty("pixels-above-lines", prScalar, "int", CAny::createInt(0));
	addProperty("pixels-below-lines", prScalar, "int", CAny::createInt(0));
	addProperty("pixels-inside-wrap", prScalar, "int", CAny::createInt(0));
	addProperty("right-margin", prScalar, "int", CAny::createInt(0));
	addProperty("wrap-mode", prScalar, "GtkWrapMode", CAny::createEnum("GtkWrapMode", Gtk::WRAP_NONE));
	addInertProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &GtkTextViewView::getText)).
		setValueSet(sigc::mem_fun(*this, &GtkTextViewView::setText));
}

void GtkTextViewView::initDesign() {
	getObject()->get_buffer()->set_text(getDesignLabel());
}

Any GtkTextViewView::getText(Property * property) {
	return CAny::createString(getObject()->get_buffer()->get_text());
}

void GtkTextViewView::setText(Property * property, Any value) {
	getObject()->get_buffer()->set_text(*value->getString());
}

// GtkIconViewView -------------------------------------------------------------

GtkIconViewView::GtkIconViewView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("column-spacing", prScalar, "int", CAny::createInt(6));
	addProperty("columns", prScalar, "int", CAny::createInt(-1));
	addProperty("item-width", prScalar, "int", CAny::createInt(-1));
	addProperty("margin", prScalar, "int", CAny::createInt(6));
	addProperty("orientation", prScalar, "GtkOrientation", CAny::createEnum("GtkOrientation", Gtk::ORIENTATION_VERTICAL));
	addProperty("row-spacing", prScalar, "int", CAny::createInt(6));
	addProperty("selection-mode", prScalar, "GtkSelectionMode", CAny::createEnum("GtkSelectionMode", Gtk::SELECTION_SINGLE));
	addProperty("spacing", prScalar, "int", CAny::createInt(0));
	addInertProperty("sample-data", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkIconViewView::setSampleData));
	addProperty("reorderable", prScalar, "bool", CAny::createBool(false));
}

void GtkIconViewView::initDesign() {
	findProperty("sample-data")->setValue(CAny::createBool(true));
}

void GtkIconViewView::setSampleData(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::IconView> iconView= getObject();
	if(value->getBool()) {
		PGdkPixbuf pixbuf= Gtk::IconTheme::get_default()->load_icon("gtk-dialog-warning", 32, Gtk::IconLookupFlags());
		Gtk::TreeModel::ColumnRecord columns;
		Gtk::TreeModelColumn<Glib::ustring> textColumn;
		Gtk::TreeModelColumn<PGdkPixbuf> pixbufColumn;
		columns.add(textColumn);
		columns.add(pixbufColumn);
		PGtkListStore model= Gtk::ListStore::create(columns);
		for(int i=0; i<9; ++i) {
			Gtk::TreeRow row= *model->append();
			row[textColumn]= "item" + ToString(i);
			row[pixbufColumn]= pixbuf;
		}
		iconView->set_model(model);
		iconView->set_text_column(textColumn);
		iconView->set_pixbuf_column(pixbufColumn);
	} else {
		iconView->set_model(PGtkListStore());
		iconView->queue_draw();
	}
}

// GtkCellViewView -------------------------------------------------------------

GtkCellViewView::GtkCellViewView() {
	addProperty("background-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkCellViewView::setBackgroundSet));
	addProperty("background-gdk", prScalar, "GdkColor", CAny::create("GdkColor", Gdk::Color())).
		setFlag(sfReadOnly);
	addInertProperty("sample-data", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkCellViewView::setSampleData));
}

void GtkCellViewView::initDesign() {
	findProperty("sample-data")->setValue(CAny::createBool(true));
}

void GtkCellViewView::setBackgroundSet(Property * property, Any value) {
	bool set= value->getBool();
	getObject()->set_background_color(Gdk::Color());
	getObject()->set_property("background-set", set);
	if(set)
		findProperty("background-gdk")->clearFlag(sfReadOnly);
	else
		findProperty("background-gdk")->setFlag(sfReadOnly);
	touch("background-gdk");
}

void GtkCellViewView::setSampleData(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::CellView> cellView= getObject();
	Gtk::TreeModel::ColumnRecord columns;
	Gtk::TreeModelColumn<Glib::ustring> column0;
	Gtk::TreeModelColumn<Glib::ustring> column1;
	Gtk::TreeModelColumn<Glib::ustring> column2;
	columns.add(column0);
	columns.add(column1);
	columns.add(column2);
	PGtkListStore model= Gtk::ListStore::create(columns);
	cellView->clear();
	cellView->set_model(model);
	if(value->getBool()) {
		Gtk::TreeRow row= *model->append();
		row[column0]= "item0";
		row[column1]= "item1";
		row[column2]= "item2";
		cellView->pack_start(column0);
		cellView->pack_start(column1);
		cellView->pack_start(column2);
		cellView->set_displayed_row(model->get_path(row));
	} else {
		cellView->queue_draw();
	}
}

// GtkTreeViewView -------------------------------------------------------------

GtkTreeViewView::GtkTreeViewView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("enable-search", prScalar, "bool", CAny::createBool(true));
	addInertProperty("fixed-height-mode", prScalar, "bool", CAny::createBool(false));
	addProperty("headers-visible", prScalar, "bool", CAny::createBool(true));
	addProperty("hover-expand", prScalar, "bool", CAny::createBool(false));
	addProperty("hover-selection", prScalar, "bool", CAny::createBool(false));
	addProperty("reorderable", prScalar, "bool", CAny::createBool(false));
	addProperty("rules-hint", prScalar, "bool", CAny::createBool(false));
	addInertProperty("selection-mode", prScalar, "GtkSelectionMode", CAny::createEnum("GtkSelectionMode", Gtk::SELECTION_SINGLE)).
		setValueGet(sigc::mem_fun(*this, &GtkTreeViewView::getSelectionMode)).
		setValueSet(sigc::mem_fun(*this, &GtkTreeViewView::setSelectionMode));
	addInertProperty("sample-data", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkTreeViewView::setSampleData));
	addProperty("rubber-banding", prScalar, "bool", CAny::createBool(false));
	addProperty("show-expanders", prScalar, "bool", CAny::createBool(true));
	addProperty("level-indentation", prScalar, "int", CAny::createInt(0));
	//addProperty("enable-grid-lines", prScalar, "GtkTreeViewGridLines", CAny::createEnum("GtkTreeViewGridLines", Gtk::TREE_VIEW_GRID_LINES_NONE));
	addProperty("enable-tree-lines", prScalar, "bool", CAny::createBool(false));
	//addInertProperty("search-entry", prEntity, "GtkEntry").
	//	setFlag(sfLinkOnly | sfWeakLink).
	//	setValueSet(sigc::mem_fun(*this, &GtkTreeViewView::setSearchEntry));
}

void GtkTreeViewView::initDesign() {
	findProperty("sample-data")->setValue(CAny::createBool(true));
}

void GtkTreeViewView::doneInstance() {
	getObject()->remove_all_columns();
	getObject()->set_model(PGtkTreeModel());
	GtkContainerViewBase::doneInstance();
}

Any GtkTreeViewView::getSelectionMode(Property * property) {
	return CAny::createEnum("GtkSelectionMode", getObject()->get_selection()->get_mode());
}

void GtkTreeViewView::setSelectionMode(Property * property, Any value) {
	getObject()->get_selection()->set_mode((Gtk::SelectionMode)value->getEnum());
}

void GtkTreeViewView::setSampleData(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::TreeView> treeView= getObject();
	treeView->remove_all_columns();
	if(value->getBool()) {
		Gtk::TreeModel::ColumnRecord columns;
		Gtk::TreeModelColumn<Glib::ustring> column0;
		Gtk::TreeModelColumn<Glib::ustring> column1;
		Gtk::TreeModelColumn<Glib::ustring> column2;
		columns.add(column0);
		columns.add(column1);
		columns.add(column2);
		PGtkListStore model= Gtk::ListStore::create(columns);
		for(int i=0; i<3; ++i) {
			Gtk::TreeRow row= *model->append();
			row[column0]= "item" + ToString(i) + "0";
			row[column1]= "item" + ToString(i) + "1";
			row[column2]= "item" + ToString(i) + "2";
		}
		treeView->set_model(model);
		treeView->append_column("Column0", column0);
		treeView->append_column("Column1", column1);
		treeView->append_column("Column2", column2);
	} else {
		treeView->set_model(PGtkTreeModel());
	}
}

void GtkTreeViewView::setSearchEntry(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::Entry> entry= value->getObject<Gtk::Entry>();
	gtk_tree_view_set_search_entry(getObject()->gobj(), entry ? entry->gobj() : NULL);
}

// GtkColorButtonView ----------------------------------------------------------

GtkColorButtonView::GtkColorButtonView() {
	addProperty("alpha", prScalar, "unsigned", CAny::createUnsigned(65535));
	addProperty("title", prScalar, "string").
		setFlag(sfAutoDefault | sfTranslatable);
	addProperty("use-alpha", prScalar, "bool", CAny::createBool(false));
	addProperty("color", prScalar, "GdkColor", CAny::create("GdkColor", Gdk::Color()));
}

// GtkColorSelectionView -------------------------------------------------------

GtkColorSelectionView::GtkColorSelectionView() {
	addProperty("has-opacity-control", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkColorSelectionView::setHasOpacityControl));
	addProperty("has-palette", prScalar, "bool", CAny::createBool(false));
	addProperty("current-alpha", prScalar, "unsigned", CAny::createUnsigned(65535)).
		setFlag(sfReadOnly);
	addProperty("current-color", prScalar, "GdkColor", CAny::create("GdkColor", Gdk::Color()));
}

void GtkColorSelectionView::setHasOpacityControl(Property * property, Any value) {
	bool has= value->getBool();
	getObject()->set_has_opacity_control(has);
	getObject()->set_current_alpha(65535);
	if(has)
		findProperty("current-alpha")->clearFlag(sfReadOnly);
	else
		findProperty("current-alpha")->setFlag(sfReadOnly);
	touch("current-alpha");
}

// GtkFileChooserButtonView ----------------------------------------------------

GtkFileChooserButtonView::GtkFileChooserButtonView() {
	addProperty("title", prScalar, "string").
		setFlag(sfAutoDefault | sfTranslatable);
	addProperty("width-chars", prScalar, "int", CAny::createInt(-1));
	addProperty("focus-on-click", prScalar, "bool", CAny::createBool(true));
}

Any GtkFileChooserButtonView::createInstance() {
	Glib::ustring backend= getPropertyModel<Glib::ustring>("file-system-backend", "");
	Gtk::FileChooserButton * widget;
	if(backend.empty())
		widget= new Gtk::FileChooserButton("", Gtk::FILE_CHOOSER_ACTION_OPEN);
	else
		widget= new Gtk::FileChooserButton("", Gtk::FILE_CHOOSER_ACTION_OPEN, backend);
	GObjectClass * oclass= G_OBJECT_GET_CLASS(widget->gobj());
	GParamSpec * property= g_object_class_find_property(oclass, "title");
	Glib::ustring title= G_PARAM_SPEC_STRING(property)->default_value;
	widget->set_title(title);
	return CAny::createObject(getType(), NewRefPtr(widget));
}

bool GtkFileChooserButtonView::isActiveWidget(Gtk::Widget * widget) {
	return false;
}

// GtkFileChooserWidgetView ----------------------------------------------------

Any GtkFileChooserWidgetView::createInstance() {
	Glib::ustring backend= getPropertyModel<Glib::ustring>("file-system-backend", "");
	if(backend.empty())
		return CAny::createObject(getType(), NewRefPtr(new Gtk::FileChooserWidget(Gtk::FILE_CHOOSER_ACTION_OPEN)));
	else
		return CAny::createObject(getType(), NewRefPtr(new Gtk::FileChooserWidget(Gtk::FILE_CHOOSER_ACTION_OPEN, backend)));
}

// GtkFontButtonView -----------------------------------------------------------

GtkFontButtonView::GtkFontButtonView() {
	addProperty("font-name", prScalar, "string").
		setFlag(sfAutoDefault);
	addProperty("show-size", prScalar, "bool", CAny::createBool(true));
	addProperty("show-style", prScalar, "bool", CAny::createBool(true));
	addProperty("title", prScalar, "string").
		setFlag(sfAutoDefault | sfTranslatable);
	addProperty("use-font", prScalar, "bool", CAny::createBool(false));
	addProperty("use-size", prScalar, "bool", CAny::createBool(false));
}

// GtkFixedView ----------------------------------------------------------------

GtkFixedView::GtkFixedView() {
	addInertProperty("has-window", prScalar, "bool", CAny::createBool(false));
}

void GtkFixedView::initInstance() {
	XYContainerView::initInstance();
	getObject()->set_has_window(true);
}

void GtkFixedView::setupContainer() {
	getContainer()->set(Children());
}

void GtkFixedView::setChildren(Property * property, Any value) {
	property->setInert(value);
	getContainer()->set(makeChildren(value));
}

// GtkLayoutView ---------------------------------------------------------------

GtkLayoutView::GtkLayoutView() {
	addProperty("size", prScalar, "CrowPoint", CAny::createPoint(Point(100, 100))).
		setValueGet(sigc::mem_fun(*this, &GtkLayoutView::getSize)).
		setValueSet(sigc::mem_fun(*this, &GtkLayoutView::setSize));
}

PGdkWindow GtkLayoutView::getExposeWindow() {
	return getObject()->get_bin_window();
}

void GtkLayoutView::setupContainer() {
	getContainer()->set(Children());
}

void GtkLayoutView::setChildren(Property * property, Any value) {
	property->setInert(value);
	getContainer()->set(makeChildren(value));
}

Any GtkLayoutView::getSize(Property * property) {
	guint width, height;
	getObject()->get_size(width, height);
	return CAny::createPoint(Point(width, height));
}

void GtkLayoutView::setSize(Property * property, Any value) {
	const Point & point= *value->getPoint();
	getObject()->set_size(point.x, point.y);
}

// GtkScrollbarView ------------------------------------------------------------

GtkScrollbarView::GtkScrollbarView() {
	addCanFocusProperty(false);
}

// GtkEventBoxView -------------------------------------------------------------

GtkEventBoxView::GtkEventBoxView() {
	addAppPaintableProperty();
	addEventProperties();
	addInertProperty("above-child", prScalar, "bool", CAny::createBool(false));
	addInertProperty("visible-window", prScalar, "bool", CAny::createBool(true));
}

// GtkDrawingAreaView ----------------------------------------------------------

GtkDrawingAreaView::GtkDrawingAreaView() {
	addEventProperties();
}

// GtkCalendarView -------------------------------------------------------------

GtkCalendarView::GtkCalendarView() {
	addCanFocusProperty(true);
	addEventProperties();
	addProperty("no-month-change", prScalar, "bool", CAny::createBool(false));
	addProperty("show-day-names", prScalar, "bool", CAny::createBool(true));
	addProperty("show-heading", prScalar, "bool", CAny::createBool(true));
	addProperty("show-week-numbers", prScalar, "bool", CAny::createBool(false));
}

// GtkArrowView ----------------------------------------------------------------

GtkArrowView::GtkArrowView() {
	addProperty("arrow-type", prScalar, "GtkArrowType", CAny::createEnum("GtkArrowType", Gtk::ARROW_RIGHT));
	addProperty("shadow-type", prScalar, "GtkShadowType", CAny::createEnum("GtkShadowType", Gtk::SHADOW_OUT));
}

Any GtkArrowView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr(new Gtk::Arrow(Gtk::ARROW_RIGHT, Gtk::SHADOW_OUT)));
}

// CustomView ------------------------------------------------------------------

CustomView::CustomView() {
	addInertProperty("type", prScalar, "string", CAny::createString(""));
	addInertProperty("meta", prScalar, "string", CAny::createString(""));
	addInertProperty("text", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
}

Any CustomView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr<Custom>());
}

// ButtonView ------------------------------------------------------------------

Any ButtonView::createInstance() {
	return CAny::createObject(getType(), NewRefPtr<Button>());
}

// WindowView ------------------------------------------------------------------

Gtk::Widget * WindowView::createDesignWindow() {
	Gtk::Widget * window= new DesignWindow();
	SetData(window, "CrowTypeHint", "CrowWindow");
	return window;
}

// GtkStatusIconView -----------------------------------------------------------

GtkStatusIconView::GtkStatusIconView() {
	addSignalsProperty();
	addProperty("blinking", prScalar, "bool", CAny::createBool(false));
	addInertProperty("visible", prScalar, "bool", CAny::createBool(false));
	addProperty("icon-name", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string");
	addInertProperty("tooltip", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
}

Any GtkStatusIconView::createInstance() {
	Glib::RefPtr<Gtk::StatusIcon> obj= Gtk::StatusIcon::create(PGdkPixbuf());
	obj->set_visible(false);
	return CAny::createObject(getType(), obj);
}

// GtkLinkButtonView -----------------------------------------------------------

GtkLinkButtonView::GtkLinkButtonView() {
	findProperty("relief")->setDefault(CAny::createEnum("GtkReliefStyle", Gtk::RELIEF_NONE));
	addProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("uri", prScalar, "string").
		setFlag(sfAutoDefault);
}

void GtkLinkButtonView::initDesign() {
	findProperty("label")->setValue(CAny::createString(getDesignLabel()));
}

Any GtkLinkButtonView::createInstance() {
	Gtk::LinkButton * widget= new Gtk::LinkButton("http://www.gtk.org", "");
	return CAny::createObject(getType(), NewRefPtr(widget));
}

// GtkRecentFilterView ---------------------------------------------------------

GtkRecentFilterView::GtkRecentFilterView() {
	addSignalsProperty();
	addProperty("name", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &GtkRecentFilterView::getName)).
		setValueSet(sigc::mem_fun(*this, &GtkRecentFilterView::setName));
	addInertProperty("add-pixbuf-formats", prScalar, "bool", CAny::createBool(false));
	addInertProperty("patterns", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("*.*")
		));
	addInertProperty("mime-types", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("")
		));
	addInertProperty("applications", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("")
		));
	addInertProperty("groups", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("")
		));
	addInertProperty("age", prScalar, "int", CAny::createInt(-1));
}

Any GtkRecentFilterView::createInstance() {
	Gtk::RecentFilter * filter= new Gtk::RecentFilter();
	filter->add_pattern("*.*");
	return CAny::createObject(getType(), NewRefPtr(filter));
}

Any GtkRecentFilterView::getName(Property * property) {
	return CAny::createString(getObject<Gtk::RecentFilter>()->get_name());
}

void GtkRecentFilterView::setName(Property * property, Any value) {
	getObject<Gtk::RecentFilter>()->set_name(*value->getString());
}

// GtkRecentChooserViewBase ----------------------------------------------------

GtkRecentChooserViewBase::GtkRecentChooserViewBase() {
	addProperty("limit", prScalar, "int", CAny::createInt(-1));
	addProperty("local-only", prScalar, "bool", CAny::createBool(true));
	addProperty("select-multiple", prScalar, "bool", CAny::createBool(false));
	addProperty("show-icons", prScalar, "bool", CAny::createBool(true));
	addProperty("show-not-found", prScalar, "bool", CAny::createBool(false));
	addProperty("show-private", prScalar, "bool", CAny::createBool(false));
	addProperty("show-tips", prScalar, "bool", CAny::createBool(false));
	addProperty("sort-type", prScalar, "GtkRecentSortType", CAny::createEnum("GtkRecentSortType", Gtk::RECENT_SORT_NONE));
	addInertProperty("filter", prEntity, "GtkRecentFilter").
		setValueSet(sigc::mem_fun(*this, &GtkRecentChooserViewBase::setFilter));
	addInertProperty("filters", prVector, "GtkRecentFilter").
		setValueSet(sigc::mem_fun(*this, &GtkRecentChooserViewBase::setFilters)).
		setElementInsert(sigc::mem_fun(*static_cast<GlibObjectView *>(this), &GlibObjectView::insertObject));
}

Glib::RefPtr<Gtk::RecentChooser> GtkRecentChooserViewBase::getObject() {
	return Glib::RefPtr<Gtk::RecentChooser>::cast_dynamic(GlibObjectView::getObject());
}

void GtkRecentChooserViewBase::initInstance() {
	GtkWidgetView::initInstance();
	initGtkRecentChooserInstance();
}

void GtkRecentChooserViewBase::initGtkRecentChooserInstance() {
	getObject()->set_limit(-1);
	getObject()->set_show_not_found(false);
	getObject()->set_show_tips(false);
}

void GtkRecentChooserViewBase::setFilter(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::RecentFilter> filter= value->getObject<Gtk::RecentFilter>();
	if(filter) {
		clearVectorProperty("filters")->setFlag(sfReadOnly);
		clearFilters();
	} else
		findProperty("filters")->clearFlag(sfReadOnly);
	touch("filters");
	getObject()->property_filter()= GetPtr(filter);
}

void GtkRecentChooserViewBase::setFilters(Property * property, Any value) {
	property->setInert(value);
	clearFilters();
	Glib::RefPtr<Gtk::RecentChooser> chooser= getObject();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<(int)vec.size(); ++i) {
		Glib::RefPtr<Gtk::RecentFilter> obj= vec[i]->getObject<Gtk::RecentFilter>();
		if(obj)
			chooser->add_filter(*GetPtr(obj));
	}
}

void GtkRecentChooserViewBase::clearFilters() {
	Glib::RefPtr<Gtk::RecentChooser> chooser= getObject();
	for(;;) {
		GSList * list= gtk_recent_chooser_list_filters(chooser->gobj());
		if(list) {
			GtkRecentFilter * filter= GTK_RECENT_FILTER(list->data);
			gtk_recent_chooser_remove_filter(chooser->gobj(), filter);
		} else
			break;
	}
}

// GtkRecentChooserDialogView --------------------------------------------------

GtkRecentChooserDialogView::GtkRecentChooserDialogView() {
	findProperty("buttons")->clearFlag(sfReadOnly | sfHidden);
	findProperty("default-response")->clearFlag(sfReadOnly | sfHidden);
	findProperty("has-separator")->
		setInertDefault(CAny::createBool(false));
}

Glib::RefPtr<Gtk::RecentChooser> GtkRecentChooserDialogView::getObject() {
	return MakeRefPtr(GlibObjectView::getObject<DesignRecentChooserDialog>()->getRecentChooser());
}

void GtkRecentChooserDialogView::initInstance() { 
	GtkDialogViewFinal::initInstance();
	initGtkRecentChooserInstance(); 
}

Gtk::Widget * GtkRecentChooserDialogView::createDesignWindow() {
	Gtk::Widget * window= new DesignRecentChooserDialog();
	SetData(window, "CrowTypeHint", "GtkRecentChooserDialog");
	return window;
}

}
