/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "application.h"

namespace CrowDesignerApp {

int Main(int argc, char ** argv) {

	Check(CROW_VERSION/1000 <= crow_version()/1000);

	std::vector<std::string> args1= Arguments(argc, argv);
	Gtk::Main gtkmain(&argc, &argv);
	std::vector<std::string> args2= Arguments(argc, argv);

	crow_enter_check("main()");
	{
		Application application(args1, args2);
		Gtk::Main::run(*application.getWindow());
	}
	crow_leave();

	return 0;
}

int TryMain(int argc, char ** argv) {
	Glib::ustring msg;
	try {
		return Main(argc, argv);
	} catch(const Glib::Exception & x) {
		msg= x.what();
	} catch(...) {
		msg= "Unknown exception";
	}
	FatalError(msg);
	return 0;
}

}

#ifdef _WIN32
	int WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
		{ return CrowDesignerApp::TryMain(__argc, __argv); }
#else
	int main(int argc, char ** argv) 
		{ return CrowDesignerApp::TryMain(argc, argv); }
#endif
