/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2007 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <stdio.h>
#include <stdlib.h>
#include <crow.h>

#include "container.h"

#ifndef DBUS_CONNECTION_FROM_G_CONNECTION
	#define _DBUS_POINTER_SHIFT(p)	((void*) (((char*)p) + sizeof (void*)))
	#define DBUS_G_CONNECTION_FROM_CONNECTION(x)	((DBusGConnection*) _DBUS_POINTER_SHIFT(x))
#endif

void OnNewConnection(
	DBusServer * server,
	DBusConnection * connection,
	void * data
) {

	static gboolean connected= FALSE;
	
	if(!connected) {

		DBusGConnection * bus= NULL;
		CrowContainer * obj= NULL;

		connected= TRUE;
		
	  dbus_connection_ref(connection);
	  dbus_connection_setup_with_g_main(connection, NULL);
	
		bus= DBUS_G_CONNECTION_FROM_CONNECTION(connection);
	
	  obj= ContainerGet();
	  dbus_g_connection_register_g_object(bus, "/CrowContainer", G_OBJECT(obj));
	
	} else {

		dbus_connection_close(connection);

	}

}

int main(int argc, char ** argv) {
	
	DBusServer * server;
	DBusError error;
	gchar * address= NULL;
	GOptionContext * context;
	GOptionEntry entries[]= {
		{ "address", 'a', 0, G_OPTION_ARG_STRING, &address, "DBus address", "A" },
		{ NULL } };
	
	gtk_init(&argc, &argv);

	context= g_option_context_new("- Crow Designer DBus Service");
	g_option_context_add_main_entries(context, entries, NULL);
	g_option_context_parse(context, &argc, &argv, NULL);

	if(!address)
		g_error("Missing DBus address\n");

	crow_enter_check("main()");
	ContainerEnter();
	
	dbus_error_init(&error);
	server= dbus_server_listen(address, &error);
	if(!server) {
		if(dbus_error_has_name(&error, DBUS_ERROR_ADDRESS_IN_USE))
			g_error("DBUS_ERROR_ADDRESS_IN_USE: %s\n", error.message);
		else
			g_error("Creating server failed: %s\n", error.message);
	}
	
	dbus_server_set_new_connection_function(server, OnNewConnection, NULL, NULL);
	dbus_server_setup_with_g_main(server, NULL);
	
	g_print("%s\n", address);
	gtk_main();
	
	dbus_server_disconnect(server);
	dbus_server_unref(server);
	ContainerLeave();
	crow_leave();

	g_option_context_free(context);
	
	return 0;
}
